# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The command to update Identity Service Feature."""


from googlecloudsdk.command_lib.anthos.common import file_parsers
from googlecloudsdk.command_lib.container.fleet import resources
from googlecloudsdk.command_lib.container.fleet.features import base
from googlecloudsdk.command_lib.container.fleet.identity_service import utils
from googlecloudsdk.core import exceptions
from googlecloudsdk.core.util import retry


# Pull out the example text so the example command can be one line without the
# py linter complaining. The docgen tool properly breaks it into multiple lines.
EXAMPLES = """\
    To apply an Identity Service configuration to a membership, run:

    $ {command} --membership=MEMBERSHIP_NAME --config=/path/to/identity-service.yaml

    To create or modify a fleet-level default membership configuration, run:

    $ {command} --fleet-default-member-config=/path/to/identity-service.yaml

    To apply an existing fleet-level default membership configuration to a membership, run:

    $ {command} --origin=fleet --membership=MEMBERSHIP_NAME
"""


class Apply(base.UpdateCommand):
  """Update the Identity Service Feature.

  This command updates the Identity Service Feature in a fleet.
  """

  detailed_help = {'EXAMPLES': EXAMPLES}

  feature_name = 'identityservice'

  @classmethod
  def Args(cls, parser):
    command_args = parser.add_group(required=True, mutex=False)
    command_args.add_argument(
        '--fleet-default-member-config',
        type=str,
        help='The path to an identity-service.yaml configuration file.',
        required=False,
    )
    per_member_config_args = command_args.add_group(required=False, mutex=False)
    resources.AddMembershipResourceArg(per_member_config_args)

    per_member_config_source = per_member_config_args.add_group(
        mutex=True, required=True
    )
    per_member_config_source.add_argument(
        '--config',
        type=str,
        help='The path to an identity-service.yaml configuration file.',
    )
    per_member_config_source.add_argument(
        '--origin',
        choices=['fleet'],
        type=str,
        help=(
            'Applies the fleet-level default membership configuration to a'
            ' membership.'
        ),
    )

  def Run(self, args):
    patch = self.messages.Feature()
    update_mask = []

    if args.config or args.origin:
      self.preparePerMemberConfigPatch(args, patch, update_mask)

    if args.fleet_default_member_config:
      self.prepareFleetDefaultMemberConfigPatch(args, patch, update_mask)

    try:
      max_wait_ms = 60000  # 60 secs
      retryer = retry.Retryer(
          max_wait_ms=max_wait_ms, exponential_sleep_multiplier=1.1
      )
      retryer.RetryOnException(
          self.Update, args=(update_mask, patch), sleep_ms=1000
      )
    except retry.MaxRetrialsException:
      raise exceptions.Error(
          'Retry limit exceeded while waiting for the {} feature to update'
          .format(self.feature.display_name)
      )

  def prepareFleetDefaultMemberConfigPatch(self, args, patch, update_mask):
    # Load the config YAML file.
    loaded_config = file_parsers.YamlConfigFile(
        file_path=args.fleet_default_member_config,
        item_type=file_parsers.LoginConfigObject,
    )

    # Create a new identity service feature spec.
    member_config = utils.parse_config(loaded_config, self.messages)

    # Add the fleet default member config to the feature patch and update
    # `update mask`.
    patch.fleetDefaultMemberConfig = (
        self.messages.CommonFleetDefaultMemberConfigSpec(
            identityservice=member_config
        )
    )
    update_mask.append('fleet_default_member_config')

  def preparePerMemberConfigPatch(self, args, patch, update_mask):
    # Get the membership the user is attempting to apply the configuration to.
    # If no membership is specified, and there is a single membership available
    # in the fleet, it would be automatically selected.
    # If no membership is specified, and there are multiple memberships
    # available in the fleet, the user would be prompted to select
    # one from the ones available in the fleet.
    membership = base.ParseMembership(
        args, prompt=True, autoselect=True, search=True
    )
    membership_spec = self.messages.MembershipFeatureSpec()
    if args.origin:
      membership_spec.origin = self.messages.Origin(
          type=self.messages.Origin.TypeValueValuesEnum('FLEET')
      )
    else:
      # Load the config YAML file.
      loaded_config = file_parsers.YamlConfigFile(
          file_path=args.config, item_type=file_parsers.LoginConfigObject
      )

      # Create a new identity service feature spec.
      member_config = utils.parse_config(loaded_config, self.messages)
      membership_spec.identityservice = member_config

    # Add the newly prepared membershipSpec to the feature patch and update
    # `update mask`.
    patch.membershipSpecs = self.hubclient.ToMembershipSpecs(
        {membership: membership_spec}
    )
    update_mask.append('membership_specs')



# -*- coding: utf-8 -*- #
# Copyright 2014 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""gcloud dns record-sets transaction describe command."""


from googlecloudsdk.api_lib.dns import transaction_util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.dns import flags


class Describe(base.DescribeCommand):
  """Describe the transaction.

  This command displays the contents of the transaction.

  ## EXAMPLES

  To look at the contents of the transaction, run:

    $ {command} --zone=MANAGED_ZONE
  """

  @staticmethod
  def Args(parser):
    flags.GetZoneArg().AddToParser(parser)

  def Run(self, args):
    api_version = 'v1'
    # If in the future there are differences between API version, do NOT use
    # this patter of checking ReleaseTrack. Break this into multiple classes.
    if self.ReleaseTrack() == base.ReleaseTrack.BETA:
      api_version = 'v1beta2'
    elif self.ReleaseTrack() == base.ReleaseTrack.ALPHA:
      api_version = 'v1alpha2'

    with transaction_util.TransactionFile(args.transaction_file) as trans_file:
      return transaction_util.ChangeFromYamlFile(
          trans_file, api_version=api_version)



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Commands for creating and manipulating service accounts."""


from googlecloudsdk.calliope import base


@base.UniverseCompatible
class ServiceAccounts(base.Group):
  """Create and manipulate service accounts.

     Create and manipulate IAM service accounts. A service account is a special
     Google account that belongs to your application or a VM, instead of to an
     individual end user. Your application uses the service account to call the
     Google API of a service, so that the users aren't directly involved.

     Note: Service accounts use client quotas for tracking usage.

     More information on service accounts can be found at:
     https://cloud.google.com/iam/docs/service-accounts
  """



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""A command that describes a API."""


from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.calliope import base


class Describe(base.DescribeCommand):
  """Describe the details of an API in discovery service."""

  @staticmethod
  def Args(parser):
    parser.add_argument(
        'api',
        help='The api_name/api_version to show the details of.')
    parser.display_info.AddFormat('json')

  def Run(self, args):
    client = apis.GetClientInstance('discovery', 'v1')
    messages = client.MESSAGES_MODULE
    api_name, api_version = args.api.split('/')
    request = messages.DiscoveryApisGetRestRequest(
        api=api_name, version=api_version)

    return client.apis.GetRest(request)




# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Update deployment group command."""


import datetime

from googlecloudsdk.api_lib.network_security.intercept_deployment_groups import api
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.network_security.intercept import deployment_group_flags
from googlecloudsdk.command_lib.util.args import labels_util


DETAILED_HELP = {
    'DESCRIPTION': """
          Update an intercept deployment group. Check the progress of deployment group update
          by using `gcloud network-security intercept-deployment-groups list`.

          For examples refer to the EXAMPLES section below.
        """,
    'EXAMPLES': """
            To update labels k1 and k2, run:

            $ {command} my-deployment-group --project=my-project --location=global --update-labels=k1=v1,k2=v2

            To remove labels k3 and k4, run:

            $ {command} my-deployment-group --project=my-project --location=global --remove-labels=k3,k4

            To clear all labels from the intercept deployment group, run:

            $ {command} my-deployment-group --project=my-project --location=global --clear-labels

            To update description to 'new description', run:

            $ {command} my-deployment-group --project=my-project --location=global --description='new description'
        """,
}


@base.DefaultUniverseOnly
@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA, base.ReleaseTrack.GA
)
class Update(base.UpdateCommand):
  """Update an Intercept Deployment Group."""

  _valid_arguments = [
      '--clear-labels',
      '--remove-labels',
      '--update-labels',
  ]

  @classmethod
  def Args(cls, parser):
    deployment_group_flags.AddDeploymentGroupResource(
        cls.ReleaseTrack(), parser
    )
    deployment_group_flags.AddMaxWait(
        parser,
        '20m',  # default to 20 minutes wait.
    )
    deployment_group_flags.AddDescriptionArg(parser)
    base.ASYNC_FLAG.AddToParser(parser)
    base.ASYNC_FLAG.SetDefault(parser, True)
    labels_util.AddUpdateLabelsFlags(parser)

  def Run(self, args):
    client = api.Client(self.ReleaseTrack())

    deployment_group = args.CONCEPTS.intercept_deployment_group.Parse()
    original = client.DescribeDeploymentGroup(deployment_group.RelativeName())
    self._validate_original_deployment_group(original)

    update_fields = {}
    labels = self._process_label_updates(client, args, original)
    if labels:
      update_fields['labels'] = labels

    if not update_fields:
      raise exceptions.MinimumArgumentException(self._valid_arguments)

    operation = client.UpdateDeploymentGroup(
        name=deployment_group.RelativeName(),
        description=getattr(args, 'description', None),
        update_fields=update_fields,
    )

    # Returns the in-progress operation if async is requested.
    if args.async_:
      # Update operations have their returned resource in YAML format by
      # default, but here we want the operation metadata to be printed.
      if not args.IsSpecified('format'):
        args.format = 'default'
      return operation

    return self._wait_for_operation(
        client,
        operation,
        deployment_group,
        datetime.timedelta(seconds=args.max_wait),
    )

  def _validate_original_deployment_group(self, original):
    if original is None:
      raise exceptions.InvalidArgumentException(
          'intercept-deployment-group',
          'Intercept deployment group does not exist.',
      )

  def _process_label_updates(self, client, args, original_dg):
    """Processes the label update request.

    Args:
      client: the client to use to make the API call.
      args: the args from the command line.
      original_dg: the original intercept deployment group.

    Returns:
      the labels we would like to update if there is any update. Otherwise,
      it returns None.
    """
    labels_diff = labels_util.Diff.FromUpdateArgs(args)
    if not labels_diff.MayHaveUpdates():
      return None

    labels = original_dg.labels
    labels_update = labels_diff.Apply(
        client.messages.InterceptDeploymentGroup.LabelsValue,
        labels,
    )
    if labels_update.needs_update:
      labels = labels_update.labels

    return labels

  def _wait_for_operation(self, client, operation, deployment_group, max_wait):
    return client.WaitForOperation(
        operation_ref=client.GetOperationRef(operation),
        message=(
            'waiting for intercept deployment group [{}] to be updated'.format(
                deployment_group.RelativeName()
            )
        ),
        has_result=False,
        max_wait=max_wait,
    )


Update.detailed_help = DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Create-preview command."""


from googlecloudsdk.api_lib.infra_manager import configmanager_util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.infra_manager import deploy_util
from googlecloudsdk.command_lib.infra_manager import flags
from googlecloudsdk.command_lib.infra_manager import resource_args
from googlecloudsdk.command_lib.util.concepts import concept_parsers


@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.GA)
class Create(base.CreateCommand):
  """Create a preview.

  This command creates a preview.
  """

  # pylint: disable=line-too-long
  detailed_help = {'EXAMPLES': """
        Create a preview named `my-preview` from a storage `my-bucket`:

          $ {command} projects/p1/locations/us-central1/previews/my-preview --gcs-source="gs://my-bucket" --input-values="project=p1,region=us-central1"

        Create a preview named `my-preview` from git repo "https://github.com/examples/repository.git", "staging/compute" folder, "mainline" branch:

          $ {command} projects/p1/locations/us-central1/previews/my-preview --git-source-repo="https://github.com/examples/repository.git"
            --git-source-directory="staging/compute" --git-source-ref="mainline"
      """}
  # pylint: enable=line-too-long

  @staticmethod
  def Args(parser):
    labels_help_text = 'Preview labels cannot be updated after creation.'
    annotations_help_text = (
        'Preview annotations cannot be updated after creation.'
    )
    flags.AddLabelsFlag(parser, labels_help_text)
    flags.AddAnnotationsFlag(parser, annotations_help_text)
    flags.AddAsyncFlag(parser)
    flags.AddDeploymentFlag(parser)
    flags.AddPreviewModeFlag(parser)
    flags.AddTerraformBlueprintFlag(parser)
    flags.AddServiceAccountFlag(parser)
    flags.AddWorkerPoolFlag(parser)
    flags.AddArtifactsGCSBucketFlag(parser)
    flags.AddProviderSourceFlag(parser)
    flags.AddTFVersionConstraintFlag(parser)

    concept_parsers.ConceptParser(
        [
            resource_args.GetLocationResourceArgSpec(
                'the location to be used as parent.'
            ),
            resource_args.GetPreviewResourceArgSpec(
                """the preview to be used as parent. It is optional and will be
              generated if not specified with a fully specified name.""",
                False,
                {'location': ''},
            ),
        ],
        command_level_fallthroughs={
            'PREVIEW.location': ['--location.location']
        },
    ).AddToParser(parser)

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Returns:
      The resulting Preview resource or, in the case that args.async_ is
        True, a long-running operation.
    """
    messages = configmanager_util.GetMessagesModule()
    preview_ref = args.CONCEPTS.preview.Parse()
    if preview_ref is not None:
      preview_full_name = preview_ref.RelativeName()
      location_full_name = preview_ref.Parent().RelativeName()
    else:
      preview_full_name = None
      location_ref = args.CONCEPTS.location.Parse()
      location_full_name = location_ref.RelativeName()

    return deploy_util.Create(
        messages,
        args.async_,
        preview_full_name,
        location_full_name,
        args.deployment,
        args.preview_mode,
        args.service_account,
        args.local_source,
        args.stage_bucket,
        args.ignore_file,
        args.artifacts_gcs_bucket,
        args.worker_pool,
        args.gcs_source,
        args.git_source_repo,
        args.git_source_directory,
        args.git_source_ref,
        args.input_values,
        args.inputs_file,
        args.labels,
        args.annotations,
        args.provider_source,
        args.tf_version_constraint,
    )



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Commands for reading and manipulating subnetworks."""


from googlecloudsdk.calliope import base


class Subnetworks(base.Group):
  """List, describe, and delete, and update Compute Engine subnetworks."""

Subnetworks.detailed_help = {
    'DESCRIPTION': """
        Read and manipulate VPC subnetworks.

        For more information about VPC, see the
        [VPC documentation](https://cloud.google.com/vpc/docs/vpc).

        See also: [Subnetworks API](https://cloud.google.com/compute/docs/reference/rest/v1/subnetworks).
    """,
}



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""`gcloud domains registrations update` command."""


from googlecloudsdk.api_lib.domains import registrations
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.domains import flags
from googlecloudsdk.command_lib.domains import resource_args
from googlecloudsdk.command_lib.domains import util
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.core import exceptions
from googlecloudsdk.core import log


@base.DefaultUniverseOnly
class Update(base.UpdateCommand):
  """Update a Cloud Domains registration.

  Update an existing registration. Currently used for updating labels only.
  Run:

    $ gcloud help alpha domains registrations configure

  to see how to change management, DNS or contact settings.

  ## EXAMPLES

  To add a label with key ``environment'' and value ``test'' for
  ``example.com'', run:

    $ {command} example.com --update-labels="project=example,environment=test"
  """

  @staticmethod
  def Args(parser):
    resource_args.AddRegistrationResourceArg(parser, 'to update')
    labels_util.AddUpdateLabelsFlags(parser)
    flags.AddAsyncFlagToParser(parser)

  def Run(self, args):
    api_version = registrations.GetApiVersionFromArgs(args)
    client = registrations.RegistrationsClient(api_version)
    args.registration = util.NormalizeResourceName(args.registration)
    registration_ref = args.CONCEPTS.registration.Parse()

    labels_update = None
    labels_diff = labels_util.Diff.FromUpdateArgs(args)
    if labels_diff.MayHaveUpdates():
      orig_resource = client.Get(registration_ref)
      labels_update = labels_diff.Apply(
          client.messages.Registration.LabelsValue,
          orig_resource.labels).GetOrNone()
    else:
      raise exceptions.Error(
          'Specify labels to update.\n'
          'Run `gcloud help alpha domains registrations configure` to see '
          'how to change management, DNS or contact settings.')
    if labels_update:
      response = client.Patch(registration_ref, labels=labels_update)
      response = util.WaitForOperation(api_version, response, args.async_)
      log.UpdatedResource(registration_ref.Name(), 'registration', args.async_)
      return response



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command group `gcloud container vmware operations`."""


from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import parser_arguments
from googlecloudsdk.command_lib.projects import util
from googlecloudsdk.core import log


@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA, base.ReleaseTrack.GA
)
class Operations(base.Group):
  """Manage Anthos on VMware long running operations."""

  @staticmethod
  def Args(parser: parser_arguments.ArgumentInterceptor):
    pass



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to update the specified Google API source."""


from googlecloudsdk.api_lib.eventarc import google_api_sources
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.eventarc import flags
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.core import log

_DETAILED_HELP = {
    'DESCRIPTION': '{description}',
    'EXAMPLES': """ \
        To update the Google API source `my-google-api-source` in location `us-central1` with destination message bus `my-message-bus`, run:

          $ {command} my-google-api-source --location=us-central1 --destination-message-bus=my-message-bus

        To update the Google API source `my-google-api-source` in location `us-central1` with `INFO` level logging, run:

          $ {command} my-google-api-source --location=us-central1 --logging-config=INFO

        To update the Google API source `my-google-api-source` in location `us-central1` with a Cloud KMS CryptoKey, run:

          $ {command} my-google-api-source --location=us-central1 --crypto-key=projects/PROJECT_ID/locations/KMS_LOCATION/keyRings/KEYRING/cryptoKeys/KEY

        """,
}


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
@base.DefaultUniverseOnly
class Update(base.UpdateCommand):
  """Update an Eventarc Google API source."""

  detailed_help = _DETAILED_HELP

  @classmethod
  def Args(cls, parser):
    flags.AddUpdateGoogleApiSourceResourceArgs(parser)
    flags.AddLoggingConfigArg(
        parser, 'The logging config of the Google API source.'
    )
    flags.AddWideScopeSubscriptionArg(parser, with_clear=True)
    flags.AddCryptoKeyArg(parser, with_clear=True)
    labels_util.AddUpdateLabelsFlags(parser)
    base.ASYNC_FLAG.AddToParser(parser)

  def Run(self, args):
    """Run the update command."""
    client = google_api_sources.GoogleApiSourceClientV1()
    google_api_source_ref = args.CONCEPTS.google_api_source.Parse()

    log.debug(
        'Updating Google API source {} for project {} in location {}'
        .format(
            google_api_source_ref.googleApiSourcesId,
            google_api_source_ref.projectsId,
            google_api_source_ref.locationsId,
        )
    )

    original_google_api_source = client.Get(google_api_source_ref)
    labels_update_result = labels_util.Diff.FromUpdateArgs(args).Apply(
        client.LabelsValueClass(), original_google_api_source.labels
    )

    update_mask = client.BuildUpdateMask(
        destination=args.IsSpecified('destination_message_bus'),
        logging_config=args.IsSpecified('logging_config'),
        crypto_key=args.IsSpecified('crypto_key'),
        clear_crypto_key=args.clear_crypto_key,
        labels=labels_update_result.needs_update,
        organization_subscription=args.IsSpecified('organization_subscription'),
        project_subscriptions=args.IsSpecified('project_subscriptions'),
        clear_project_subscriptions=args.clear_project_subscriptions,
    )

    operation = client.Patch(
        google_api_source_ref,
        client.BuildGoogleApiSource(
            google_api_source_ref=google_api_source_ref,
            destination_ref=args.CONCEPTS.destination_message_bus.Parse(),
            logging_config=args.logging_config,
            crypto_key_name=args.crypto_key,
            labels=labels_update_result.GetOrNone(),
            organization_subscription=args.organization_subscription
            if args.IsSpecified('organization_subscription')
            else None,
            project_subscriptions=args.project_subscriptions,
        ),
        update_mask,
    )

    if args.async_:
      return operation
    return client.WaitFor(operation, 'Updating', google_api_source_ref)



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The command group for cloud dataproc workflow templates."""


from googlecloudsdk.calliope import base


@base.DefaultUniverseOnly
class WorkflowTemplates(base.Group):
  r"""Create and manage Dataproc workflow templates.

  Create and manage Dataproc workflow templates.

  ## EXAMPLES

  To create a workflow template, run:

    $ {command} create my_template

  To instantiate a workflow template, run:

    $ {command} instantiate my_template

  To instantiate a workflow template from a file, run:

    $ {command} instantiate-from-file --file template.yaml

  To delete a workflow template, run:

    $ {command} delete my_template

  To view the details of a workflow template, run:

    $ {command} describe my_template

  To see the list of all workflow templates, run:

    $ {command} list

  To remove a job from a workflow template, run:

    $ {command} remove-job my_template --step-id id

  To update managed cluster in a workflow template, run:

    $ {command} set-managed-cluster my_template --num-masters 5

  To update cluster selector in a workflow template, run:

    $ {command} set-cluster-selector my_template \
        --cluster-labels environment=prod

  """

  pass



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The gcloud firestore operations wait command."""


from googlecloudsdk.api_lib.firestore import operations
from googlecloudsdk.calliope import base


@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA
)
class Wait(base.Command):
  """Waits a Cloud Firestore admin operation to complete."""

  detailed_help = {
      'EXAMPLES':
          """\
          To wait a Cloud Firestore admin operation `exampleOperationId` to
          complete, run:

            $ {command} exampleOperationId
      """
  }

  @staticmethod
  def Args(parser):
    """Register flags for this command."""
    parser.add_argument(
        'name',
        type=str,
        default=None,
        help="""
        The unique name of the Operation to retrieve, formatted as full resource
        path:

          projects/my-app-id/databases/(default)/operations/foo
        """)

  def Run(self, args):
    return operations.WaitForOperationWithName(args.name)



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command group for Config Management Feature."""


from googlecloudsdk.calliope import base as calliope_base


class Configmanagement(calliope_base.Group):
  """Use the Config Management feature."""

  category = calliope_base.COMPUTE_CATEGORY
  # Config Sync link breaks in docstring because it does not fit on 1 line.
  # Pylint disallows long lines, even for a raw docstring, and the use of
  # backslashes.
  detailed_help = {
      'DESCRIPTION': """
Manage
[Config Sync](https://docs.cloud.google.com/kubernetes-engine/config-sync/docs/how-to/installing-config-sync)
using the Config Management feature.

To manage Policy Controller, use `gcloud container fleet policycontroller`
instead.

Hierarchy Controller is no longer available to install. If Hierarchy Controller
is still configured, Config Sync upgrades are blocked. To upgrade Config Sync,
disable Hierarchy Controller following
https://docs.cloud.google.com/kubernetes-engine/config-sync/docs/how-to/migrate-hierarchy-controller.
"""
  }



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""The command group for the Endpoints V1 CLI."""


from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.calliope import base


@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA,
    base.ReleaseTrack.BETA,
    base.ReleaseTrack.GA,
)
class Endpoints(base.Group):
  """Create, enable and manage API services.

  Google Service Management is an infrastructure service of Google Cloud
  Platform that manages other APIs and services, including Google's own Cloud
  Platform services and their APIs, and services created using Google Cloud
  Endpoints.

  More information on Service Management can be found here:
  https://cloud.google.com/service-management and detailed documentation can be
  found here: https://cloud.google.com/service-management/docs/
  """

  category = base.API_PLATFORM_AND_ECOSYSTEMS_CATEGORY

  def Filter(self, context, args):
    """Context() is a filter function that can update the context.

    Args:
      context: The current context.
      args: The argparse namespace that was specified on the CLI or API.

    Returns:
      The updated context.
    """
    # TODO(b/190533981):  Determine if command group works with project number
    base.RequireProjectID(args)
    base.DisableUserProjectQuota()
    context['servicemanagement-v1'] = apis.GetClientInstance(
        'servicemanagement', 'v1')
    context['servicemanagement-v1-messages'] = apis.GetMessagesModule(
        'servicemanagement', 'v1')

    return context



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Delete batches command."""


from googlecloudsdk.api_lib.dataproc import dataproc as dp
from googlecloudsdk.api_lib.dataproc import util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.dataproc import flags
from googlecloudsdk.core import log
from googlecloudsdk.core.console import console_io


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
class Delete(base.DeleteCommand):
  """Delete a batch job."""
  detailed_help = {
      'EXAMPLES':
          """\
          To delete a batch job, run:

            $ {command} my-batch-job --region=us-central1
          """
    }

  @staticmethod
  def Args(parser):
    dataproc = dp.Dataproc(base.ReleaseTrack.GA)

    flags.AddAsync(parser)
    flags.AddBatchResourceArg(parser, 'delete', dataproc.api_version)

  def Run(self, args):
    dataproc = dp.Dataproc(base.ReleaseTrack.GA)
    messages = dataproc.messages

    batch_id = args.CONCEPTS.batch.Parse()

    request = messages.DataprocProjectsLocationsBatchesDeleteRequest(
        name=batch_id.RelativeName())

    console_io.PromptContinue(
        message="The batch '{}' will be deleted.".format(batch_id.Name()),
        cancel_on_no=True,
        cancel_string='Deletion aborted by user.')

    dataproc.client.projects_locations_batches.Delete(request)

    if args.async_:
      log.status.Print("Deleting batch '{}'.".format(batch_id.Name()))
      return

    def _GetJob(job_ref):
      return dataproc.client.projects_locations_batches.Get(
          dataproc.messages.DataprocProjectsLocationsBatchesGetRequest(
              name=job_ref))

    util.WaitForResourceDeletion(
        _GetJob, batch_id.RelativeName(), message='Waiting for batch deletion')

    log.DeletedResource(batch_id.RelativeName())



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for describing interconnect attachment groups."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute.interconnects.attachments.groups import client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.interconnects.attachments.groups import flags
from googlecloudsdk.core import properties

DETAILED_HELP = {
    'DESCRIPTION': """\
        *{command}* is used to describe interconnect attachment groups.

        For an example, refer to the *EXAMPLES* section below.
        """,
    'EXAMPLES': """\
        To describe an interconnect attachment group, run:

          $ {command} example-attachment-group
        """,
}


@base.UniverseCompatible
@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA,
    base.ReleaseTrack.BETA,
    base.ReleaseTrack.GA,
    base.ReleaseTrack.PREVIEW,
)
class Describe(base.DescribeCommand):
  """Describe a Compute Engine interconnect attachment group.

  *{command}* displays all data associated with Compute Engine
  interconnect attachment group in a project.
  """

  ATTACHMENT_GROUP_ARG = None

  @classmethod
  def Args(cls, parser):
    cls.ATTACHMENT_GROUP_ARG = flags.InterconnectAttachmentGroupArgument()
    cls.ATTACHMENT_GROUP_ARG.AddArgument(parser, operation_type='describe')

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    ref = self.ATTACHMENT_GROUP_ARG.ResolveAsResource(args, holder.resources)
    project = properties.VALUES.core.project.GetOrFail()

    attachment_group = client.InterconnectAttachmentGroup(
        ref, project, compute_client=holder.client
    )
    return attachment_group.Describe()


Describe.detailed_help = DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2014 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""operations describe command."""


from apitools.base.py import exceptions as apitools_exceptions

from googlecloudsdk.api_lib.deployment_manager import dm_api_util
from googlecloudsdk.api_lib.deployment_manager import dm_base
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions


@dm_base.UseDmApi(dm_base.DmApiVersion.V2)
class Describe(base.DescribeCommand, dm_base.DmCommand):
  """Provide information about an operation.

  This command prints out all available details about an operation.
  """

  detailed_help = {
      'EXAMPLES': """\
          To display information about an operation, run:

            $ {command} operation-name
          """,
  }

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command.

    Args:
      parser: An argparse parser that you can use to add arguments that go
          on the command line after this command. Positional arguments are
          allowed.
    """
    parser.add_argument('operation_name', help='Operation name.')

  def Run(self, args):
    """Run 'operations describe'.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
          with.

    Returns:
      The requested operation.

    Raises:
      HttpException: An http error response was received while executing api
          request.
    """
    try:
      return self.client.operations.Get(
          self.messages.DeploymentmanagerOperationsGetRequest(
              project=dm_base.GetProject(),
              operation=args.operation_name,
          )
      )
    except apitools_exceptions.HttpError as error:
      raise exceptions.HttpException(error, dm_api_util.HTTP_ERROR_FORMAT)



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Implementation of gcloud dataflow jobs list command."""


from googlecloudsdk.api_lib.dataflow import apis
from googlecloudsdk.api_lib.dataflow import job_display
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.dataflow import dataflow_util
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core.resource import resource_filter
from googlecloudsdk.core.util import times


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
class List(base.ListCommand):
  """Lists all jobs in a particular project, optionally filtered by region.

  By default, 100 jobs in the current project are listed; this can be overridden
  with the gcloud --project flag, and the --limit flag.

  Using the --region flag will only list jobs from the given regional endpoint.

  ## EXAMPLES

  Filter jobs with the given name:

    $ {command} --filter="name=my-wordcount"

  List jobs with from a given region:

    $ {command} --region="europe-west1"

  List jobs created this year:

    $ {command} --created-after=2018-01-01

  List jobs created more than a week ago:

    $ {command} --created-before=-P1W
  """

  @staticmethod
  def Args(parser):
    """Register flags for this command."""

    base.ASYNC_FLAG.RemoveFromParser(parser)
    # Set manageable limits on number of jobs that are listed.
    base.LIMIT_FLAG.SetDefault(parser, 100)
    base.PAGE_SIZE_FLAG.SetDefault(parser, 20)

    # Flags for filtering jobs.
    parser.add_argument(
        '--status',
        choices={
            'all':
                ('Returns running jobs first, ordered on creation timestamp, '
                 'then, returns all terminated jobs ordered on the termination '
                 'timestamp.'),
            'terminated':
                ('Filters the jobs that have a terminated state, ordered on '
                 'the termination timestamp. Example terminated states: Done, '
                 'Updated, Cancelled, etc.'),
            'active':
                ('Filters the jobs that are running ordered on the creation '
                 'timestamp.'),
        },
        help='Filter the jobs to those with the selected status.')
    parser.add_argument(
        '--created-after',
        type=arg_parsers.Datetime.Parse,
        help=('Filter the jobs to those created after the given time. '
              'See $ gcloud topic datetimes for information on time formats. '
              'For example, `2018-01-01` is the first day of the year, and '
              '`-P2W` is 2 weeks ago.'))
    parser.add_argument(
        '--created-before',
        type=arg_parsers.Datetime.Parse,
        help=('Filter the jobs to those created before the given time. '
              'See $ gcloud topic datetimes for information on time formats.'))
    parser.add_argument(
        '--region',
        metavar='REGION',
        help=(
            'Only resources from the given region are queried. '
            'If not provided, an attempt will be made to query from all '
            'available regions. In the event of an outage, jobs from certain '
            'regions may not be available.'))

    parser.display_info.AddFormat("""
          table(
            id:label=JOB_ID,
            name:label=NAME,
            type:label=TYPE,
            creationTime.yesno(no="-"),
            state,
            location:label=REGION
          )
     """)
    parser.display_info.AddUriFunc(dataflow_util.JobsUriFunc)

  def Run(self, args):
    """Runs the command.

    Args:
      args: All the arguments that were provided to this command invocation.

    Returns:
      An iterator over Job messages.
    """
    if args.filter:
      filter_expr = resource_filter.Compile(args.filter)

      def EvalFilter(x):
        return (filter_expr.Evaluate(job_display.DisplayInfo(x)) and
                _JobFilter(args)(x))

      filter_pred = EvalFilter
    else:
      filter_pred = _JobFilter(args)

    project_id = properties.VALUES.core.project.Get(required=True)
    jobs = self._JobSummariesForProject(project_id, args, filter_pred)

    return [job_display.DisplayInfo(job) for job in jobs]

  def _JobSummariesForProject(self, project_id, args, filter_predicate):
    """Get the list of job summaries that match the predicate.

    Args:
      project_id: The project ID to retrieve
      args: parsed command line arguments
      filter_predicate: The filter predicate to apply

    Returns:
      An iterator over all the matching jobs.
    """
    request = None
    service = None
    status_filter = self._StatusArgToFilter(args.status, args.region)
    if args.region:
      request = apis.Jobs.LIST_REQUEST(
          projectId=project_id, location=args.region, filter=status_filter)
      service = apis.Jobs.GetService()
    else:
      log.status.Print(
          '`--region` not set; getting jobs from all available regions. ' +
          'Some jobs may be missing in the event of an outage. ' +
          'https://cloud.google.com/dataflow/docs/concepts/regional-endpoints')
      request = apis.Jobs.AGGREGATED_LIST_REQUEST(
          projectId=project_id, filter=status_filter)
      service = apis.GetClientInstance().projects_jobs

    return dataflow_util.YieldFromList(
        project_id=project_id,
        region_id=args.region,
        service=service,
        request=request,
        limit=args.limit,
        batch_size=args.page_size,
        field='jobs',
        batch_size_attribute='pageSize',
        predicate=filter_predicate)

  def _StatusArgToFilter(self, status, region=None):
    """Return a string describing the job status.

    Args:
      status: The job status enum
      region: The region argument, to select the correct wrapper message.

    Returns:
      string describing the job status
    """

    filter_value_enum = None
    if region:
      filter_value_enum = (
          apis.GetMessagesModule().DataflowProjectsLocationsJobsListRequest
          .FilterValueValuesEnum)
    else:
      filter_value_enum = (
          apis.GetMessagesModule().DataflowProjectsJobsAggregatedRequest
          .FilterValueValuesEnum)

    value_map = {
        'all': filter_value_enum.ALL,
        'terminated': filter_value_enum.TERMINATED,
        'active': filter_value_enum.ACTIVE,
    }
    return value_map.get(status, filter_value_enum.ALL)


class _JobFilter(object):
  """Predicate for filtering jobs."""

  def __init__(self, args):
    """Create a _JobFilter from the given args.

    Args:
      args: The argparse.Namespace containing the parsed arguments.
    """
    self.preds = []
    if args.created_after or args.created_before:
      self._ParseTimePredicate(args.created_after, args.created_before)

  def __call__(self, job):
    return all([pred(job) for pred in self.preds])

  def _ParseTimePredicate(self, after, before):
    """Return a predicate for filtering jobs by their creation time.

    Args:
      after: Only return true if the job was created after this time.
      before: Only return true if the job was created before this time.
    """
    if after and (not before):
      self.preds.append(lambda x: times.ParseDateTime(x.createTime) > after)
    elif (not after) and before:
      self.preds.append(lambda x: times.ParseDateTime(x.createTime) <= before)
    elif after and before:

      def _Predicate(x):
        create_time = times.ParseDateTime(x.createTime)
        return after < create_time and create_time <= before

      self.preds.append(_Predicate)



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Delete endpoint group association command."""


import datetime

from googlecloudsdk.api_lib.network_security.mirroring_endpoint_group_associations import api
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.network_security import endpoint_group_association_flags

DETAILED_HELP = {
    'DESCRIPTION': """
          Delete a mirroring endpoint group association. Check the progress of deletion
          by using `gcloud network-security mirroring-endpoint-group-associations list`.

          For more examples, refer to the EXAMPLES section below.

        """,
    'EXAMPLES': """
            To delete a mirroring endpoint group association called `my-association`, in project ID `my-project`, run:

            $ {command} my-association --project=my-project --location=global

            OR

            $ {command} projects/my-project/locations/global/mirroringEndpointGroupAssociations/my-association

        """,
}


@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA,
                    base.ReleaseTrack.GA)
class Delete(base.DeleteCommand):
  """Delete a Mirroring Endpoint Group Association."""

  @classmethod
  def Args(cls, parser):
    endpoint_group_association_flags.AddEndpointGroupAssociationResource(
        cls.ReleaseTrack(), parser
    )
    endpoint_group_association_flags.AddMaxWait(
        parser, '20m'  # default to 20 minutes wait.
    )
    base.ASYNC_FLAG.AddToParser(parser)
    base.ASYNC_FLAG.SetDefault(parser, True)

  def Run(self, args):
    client = api.Client(self.ReleaseTrack())

    association = args.CONCEPTS.mirroring_endpoint_group_association.Parse()

    is_async = args.async_
    max_wait = datetime.timedelta(seconds=args.max_wait)

    operation = client.DeleteEndpointGroupAssociation(
        name=association.RelativeName(),
    )
    # Return the in-progress operation if async is requested.
    if is_async:
      # Delete operations have their returned resource in YAML format by
      # default, but here we want the operation metadata to be printed.
      if not args.IsSpecified('format'):
        args.format = 'default'
      return operation
    return client.WaitForOperation(
        operation_ref=client.GetOperationRef(operation),
        message=(
            'waiting for mirroring endpoint group association [{}] to be'
            ' deleted'.format(association.RelativeName())
        ),
        has_result=False,
        max_wait=max_wait,
    )


Delete.detailed_help = DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to create a channel connection."""


from googlecloudsdk.api_lib.eventarc import channel_connections
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.eventarc import flags
from googlecloudsdk.core import log

_DETAILED_HELP = {
    'DESCRIPTION':
        '{description}',
    'EXAMPLES':
        """\
        To create a new channel connection ``my-channel-connection'' for channel ``my-channel'' with activation token ``channel-activation-token'', run:

          $ {command} my-channel-connection --channel=my-channel --activation-token=channel-activation-token
        """,
}

ACTIVATION_TOKEN_FLAG = base.Argument(
    '--activation-token',
    dest='activation_token',
    help="""Activation token for the specified channel.""",
    required=True)

CHANNEL_FLAG = base.Argument(
    '--channel',
    dest='channel',
    help="""Subscriber channel for which to create the channel connection. This argument should be the full channel name, including project, location and the channel id. """,
    required=True)


@base.ReleaseTracks(base.ReleaseTrack.GA)
@base.DefaultUniverseOnly
class Create(base.CreateCommand):
  """Create an Eventarc channel connection."""

  detailed_help = _DETAILED_HELP

  @classmethod
  def Args(cls, parser):
    flags.AddChannelConnectionResourceArg(parser,
                                          'Channel connection to create.')
    CHANNEL_FLAG.AddToParser(parser)
    ACTIVATION_TOKEN_FLAG.AddToParser(parser)
    flags.AddLabelsArg(
        parser, help_text='Labels to apply to the channel connection.'
    )
    base.ASYNC_FLAG.AddToParser(parser)

  def Run(self, args):
    """Run the create command."""
    client = channel_connections.ChannelConnectionClientV1()
    channel_connection_ref = args.CONCEPTS.channel_connection.Parse()

    project_name = channel_connection_ref.Parent().Parent().Name()
    location_name = channel_connection_ref.Parent().Name()
    log.debug('Creating channel {} for project {} in location {}'.format(
        channel_connection_ref.Name(), project_name, location_name))
    operation = client.Create(
        channel_connection_ref,
        client.BuildChannelConnection(
            channel_connection_ref,
            channel=args.channel,
            activation_token=args.activation_token,
            labels=args.labels,
        ),
    )

    if args.async_:
      return operation
    return client.WaitFor(operation, 'Creating', channel_connection_ref)



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Delete attachment command."""


import datetime

from googlecloudsdk.api_lib.network_security.firewall_attachments import attachment_api
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.network_security import attachment_flags

DETAILED_HELP = {
    'DESCRIPTION': """
          Delete a firewall attachment. Check the progress of attachment deletion
          by using `gcloud network-security firewall-attachments list`.

          For more examples, refer to the EXAMPLES section below.

        """,
    'EXAMPLES': """
            To delete a firewall attachment called `my-attachment`, in zone
            `us-central1-a` and project my-proj, run:

            $ {command} my-attachment --zone=us-central1-a --project=my-proj

            $ {command} projects/my-proj/locations/us-central1-a/firewallAttachments/my-attachment

        """,
}


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class Delete(base.DeleteCommand):
  """Delete a Firewall attachment."""

  @classmethod
  def Args(cls, parser):
    attachment_flags.AddAttachmentResource(cls.ReleaseTrack(), parser)
    attachment_flags.AddMaxWait(parser, '60m')  # default to 60 minutes wait.
    base.ASYNC_FLAG.AddToParser(parser)
    base.ASYNC_FLAG.SetDefault(parser, True)

  def Run(self, args):
    client = attachment_api.Client(self.ReleaseTrack())

    attachment = args.CONCEPTS.firewall_attachment.Parse()

    is_async = args.async_
    max_wait = datetime.timedelta(seconds=args.max_wait)

    operation = client.DeleteAttachment(
        name=attachment.RelativeName(),
    )
    # Return the in-progress operation if async is requested.
    if is_async:
      # Delete operations have no format by default,
      # but here we want the operation metadata to be printed.
      if not args.IsSpecified('format'):
        args.format = 'default'
      return operation
    return client.WaitForOperation(
        operation_ref=client.GetOperationRef(operation),
        message=(
            'waiting for firewall attachment [{}] to be deleted'
            .format(attachment.RelativeName())
        ),
        has_result=True,
        max_wait=max_wait,
    )


Delete.detailed_help = DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Approve a single tenant HSM instance proposal."""


import ast

from googlecloudsdk.api_lib.cloudkms import base as cloudkms_base
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.kms import pems
from googlecloudsdk.command_lib.kms import resource_args
from googlecloudsdk.core import exceptions as core_exceptions
from googlecloudsdk.core import log
from googlecloudsdk.core.util import files


def tuple_list_type(arg):
  try:
    return ast.literal_eval(arg)
  except ValueError as e:
    raise exceptions.BadArgumentException(
        '--challenge_replies',
        'Error while attempting to parse challenge replies: {}'.format(e),
    )


def _parse_challenge_replies(messages, challenge_reply_tuples):
  """Parses file paths from tuples into a list of ChallengeReply messages."""
  challenge_replies = []
  if not challenge_reply_tuples:
    return challenge_replies

  for challenge_reply in challenge_reply_tuples:
    if not isinstance(challenge_reply, tuple) or len(challenge_reply) != 2:
      raise exceptions.BadArgumentException(
          'challenge replies',
          'Each challenge reply must be a tuple of (signed_challenge_file,'
          ' public_key_file).',
      )
    signed_challenge_file, public_key_file = challenge_reply

    if not signed_challenge_file:
      raise exceptions.BadArgumentException(
          'challenge replies',
          'signed_challenge_file must be specified.',
      )
    if not public_key_file:
      raise exceptions.BadArgumentException(
          'challenge replies',
          'public_key_file must be specified.',
      )

    try:
      public_key = pems.GetPemPublicKey(public_key_file)
    except Exception as e:
      raise exceptions.BadArgumentException(
          'challenge replies',
          'Error while attempting to read public key file {}: {}'.format(
              public_key_file, e
          ),
      )
    try:
      signed_challenge = files.ReadBinaryFileContents(signed_challenge_file)
    except Exception as e:
      raise exceptions.BadArgumentException(
          'challenge replies',
          'Error while attempting to read signed challenge file {}: {}'.format(
              signed_challenge_file, e
          ),
      )
    challenge_replies.append(
        messages.ChallengeReply(
            signedChallenge=signed_challenge,
            publicKeyPem=public_key,
        )
    )
  return challenge_replies


@base.DefaultUniverseOnly
class Approve(base.Command):
  r"""Approve a single tenant HSM instance proposal.

  ## EXAMPLES

  The following command approves a single tenant HSM instance proposal with
  quorum challenge replies:

  $ {command} projects/my-project/locations/us-east1/singleTenantHsmInstances/
  my_sthi/proposals/my_proposal \
  --quorum-challenge-replies="[('signed_challenge_1.txt','public_key_1.pem'),
  ('signed_challenge_2.txt','public_key_2.pem'),
  ('signed_challenge_3.txt','public_key_3.pem')]"

  To approve a proposal with required challenges and quorum challenges:

  $ {command} projects/my-project/locations/us-east1/singleTenantHsmInstances/
  my_sthi/proposals/my_proposal \
  --required-challenge-replies="[('required_challenge.txt','public_key_1.pem')]" \
  --quorum-challenge-replies="[('quorum_challenge_1.txt','public_key_2.pem'),
  ('quorum_challenge_2.txt','public_key_3.pem')]"
  """

  @staticmethod
  def Args(parser):
    resource_args.AddKmsSingleTenantHsmInstanceProposalResourceArgForKMS(
        parser, True, 'single_tenant_hsm_instance_proposal'
    )
    quorum_group = parser.add_group(
        required=True, help='Approval payload for the proposal.'
    )
    quorum_group.add_argument(
        '--quorum-challenge-replies',
        type=tuple_list_type,
        help=(
            'The challenge replies to approve the proposal. Challenge replies '
            'can be sent across multiple requests. Each tuple should be '
            '("signed_challenge_file", "public_key_file").'
        ),
    )
    quorum_group.add_argument(
        '--required-challenge-replies',
        type=tuple_list_type,
        help=(
            'A list of tuples, each containing the file paths for a required '
            'challenge reply. Each tuple should be '
            '("signed_challenge_file", "public_key_file").'
        ),
    )

  def ApproveRequest(self, args):
    messages = cloudkms_base.GetMessagesModule()
    single_tenant_hsm_instance_proposal_ref = (
        args.CONCEPTS.single_tenant_hsm_instance_proposal.Parse()
    )
    kms_client = cloudkms_base.GetClientInstance()
    proposal = kms_client.projects_locations_singleTenantHsmInstances_proposals.Get(
        messages.CloudkmsProjectsLocationsSingleTenantHsmInstancesProposalsGetRequest(
            name=single_tenant_hsm_instance_proposal_ref.RelativeName()
        )
    )

    approve_request = messages.ApproveSingleTenantHsmInstanceProposalRequest()

    if proposal.quorumParameters is not None:
      if args.quorum_challenge_replies is None:
        raise exceptions.RequiredArgumentException(
            '--quorum-challenge-replies',
            '--quorum-challenge-replies must be specified for proposals'
            ' requiring quorum parameters.',
        )
      if args.required_challenge_replies is not None:
        raise exceptions.BadArgumentException(
            '--required-challenge-replies',
            'This argument cannot be used with proposals requiring quorum'
            ' parameters.',
        )
      challenge_replies = _parse_challenge_replies(
          messages, args.quorum_challenge_replies
      )
      approve_request.quorumReply = messages.QuorumReply(
          challengeReplies=challenge_replies
      )
    elif proposal.requiredActionQuorumParameters is not None:
      if (
          args.required_challenge_replies is None
          and args.quorum_challenge_replies is None
      ):
        raise exceptions.RequiredArgumentException(
            '--required-challenge-replies and --quorum-challenge-replies',
            'At least one of --required-challenge-replies and'
            ' --quorum-challenge-replies must be specified for proposals'
            ' requiring required action quorum parameters.',
        )
      required_challenge_replies = _parse_challenge_replies(
          messages, args.required_challenge_replies
      )
      quorum_challenge_replies = _parse_challenge_replies(
          messages, args.quorum_challenge_replies
      )
      approve_request.requiredActionQuorumReply = (
          messages.RequiredActionQuorumReply(
              requiredChallengeReplies=required_challenge_replies,
              quorumChallengeReplies=quorum_challenge_replies,
          )
      )
    else:
      raise core_exceptions.Error(
          'Unsupported or missing approval parameters in proposal: {}'.format(
              proposal.name
          )
      )

    req = messages.CloudkmsProjectsLocationsSingleTenantHsmInstancesProposalsApproveRequest(
        name=single_tenant_hsm_instance_proposal_ref.RelativeName(),
        approveSingleTenantHsmInstanceProposalRequest=approve_request,
    )
    return req

  def Run(self, args):
    """Approve a single tenant HSM instance proposal."""
    client = cloudkms_base.GetClientInstance()
    result = (
        client.projects_locations_singleTenantHsmInstances_proposals.Approve(
            self.ApproveRequest(args)
        )
    )
    if result is not Exception:
      log.status.Print('Approved proposal successfully.')
    return result



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The gcloud datastore emulator group."""


from googlecloudsdk.calliope import actions
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.emulators import datastore_util
from googlecloudsdk.command_lib.emulators import flags
from googlecloudsdk.command_lib.emulators import util
from googlecloudsdk.command_lib.util import java


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
class Datastore(base.Group):
  """Manage your local datastore emulator.

  This set of commands allows you to start and use a local datastore emulator.
  """

  detailed_help = {
      'EXAMPLES': """\
          To start a local datastore emulator, run:

            $ {command} start
          """,
  }

  @staticmethod
  def Args(parser):
    flags.AddDataDirFlag(parser, datastore_util.DATASTORE)

  def Filter(self, context, args):
    java.RequireJavaInstalled(datastore_util.DATASTORE_TITLE, min_version=8)
    util.EnsureComponentIsInstalled('cloud-datastore-emulator',
                                    datastore_util.DATASTORE_TITLE)
    if not args.data_dir:
      args.data_dir = datastore_util.GetDataDir()



# -*- coding: utf-8 -*- #
# Copyright 2018 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Set the IAM policy for a Binary Authorization policy."""


from googlecloudsdk.api_lib.container.binauthz import apis
from googlecloudsdk.api_lib.container.binauthz import iam
from googlecloudsdk.api_lib.container.binauthz import util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.iam import iam_util


@base.DefaultUniverseOnly
class SetIamPolicy(base.Command):
  """Set the IAM policy for a Binary Authorization policy.

  See https://cloud.google.com/iam/docs/managing-policies for details of
  the policy file format and contents.

  ## EXAMPLES
  The following command will read an IAM policy defined in a JSON file
  'iam_policy.json' and set it for the current project's Binary Authorization
  policy:

    $ {command} iam_policy.json
  """
  # The above text is based on output of
  # iam_util.GetDetailedHelpForSetIamPolicy.

  @classmethod
  def Args(cls, parser):
    parser.add_argument(
        'policy_file',
        help=('The JSON or YAML '
              'file containing the IAM policy.'))

  def Run(self, args):
    api_version = apis.GetApiVersion(self.ReleaseTrack())
    client = iam.Client(api_version)
    policy_ref = util.GetPolicyRef()

    policy, _ = iam_util.ParseYamlOrJsonPolicyFile(args.policy_file,
                                                   client.messages.IamPolicy)

    result = client.Set(policy_ref, policy)
    iam_util.LogSetIamPolicy(policy_ref.Name(), 'policy')
    return result



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Removes an invoker binding from the IAM policy of a Google Cloud Function."""


from googlecloudsdk.api_lib.functions.v1 import util as api_util_v1
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import parser_extensions
from googlecloudsdk.command_lib.functions import flags
from googlecloudsdk.command_lib.functions import util
from googlecloudsdk.command_lib.functions.v2.remove_invoker_policy_binding import command as command_v2
from googlecloudsdk.command_lib.iam import iam_util


_DETAILED_HELP = {
    'DESCRIPTION': """\
      Removes the invoker role IAM policy binding that allows the specified
      member to invoke the specified function.

      For Cloud Functions (1st gen), this removes the Cloud Functions Invoker
      binding from the IAM policy of the specified function.

      For Cloud Functions (2nd gen), this removes the Cloud Run Invoker binding
      from the IAM policy of the specified function's underlying Cloud Run
      service.
      """,
    'EXAMPLES': """\
          To remove the invoker role policy binding for `FUNCTION-1` for member
          `MEMBER-1` run:

            $ {command} FUNCTION-1 --member=MEMBER-1
          """,
}


@base.ReleaseTracks(base.ReleaseTrack.GA)
class RemoveInvokerPolicyBinding(util.FunctionResourceCommand):
  """Removes an invoker binding from the IAM policy of a Google Cloud Function.

  This command applies to Cloud Functions 2nd gen only.
  """

  detailed_help = _DETAILED_HELP

  @staticmethod
  def Args(parser):
    """Registers flags for this command."""
    flags.AddFunctionResourceArg(parser, 'to remove the invoker binding from')
    flags.AddGen2Flag(parser, 'to remove the invoker binding from', hidden=True)
    iam_util.AddMemberFlag(parser, 'to remove from the IAM policy', False)

  def _RunV1(self, args: parser_extensions.Namespace):
    return api_util_v1.RemoveFunctionIamPolicyBindingIfFound(
        args.CONCEPTS.name.Parse().RelativeName(),
        member=args.member,
        role='roles/cloudfunctions.invoker',
    )

  def _RunV2(self, args: parser_extensions.Namespace):
    return command_v2.Run(args, self.ReleaseTrack())


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class RemoveInvokerPolicyBindingBeta(RemoveInvokerPolicyBinding):
  """Removes an invoker binding from the IAM policy of a Google Cloud Function.

  This command applies to Cloud Functions 2nd gen only.
  """


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class RemoveInvokerPolicyBindingAlpha(RemoveInvokerPolicyBindingBeta):
  """Removes an invoker binding from the IAM policy of a Google Cloud Function.

  This command applies to Cloud Functions 2nd gen only.
  """



# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Command for deleting a role."""


from googlecloudsdk.api_lib.iam import util
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.iam import flags
from googlecloudsdk.command_lib.iam import iam_util


DETAILED_HELP = {
    'EXAMPLES':
        """\
          To delete the role ``ProjectUpdater'' of the organization ``1234567'',
          run:

            $ {command} ProjectUpdater --organization=1234567

          To delete the role ``ProjectUpdater'' of the project ``myproject'',
          run:

            $ {command} ProjectUpdater --project=myproject
        """
}


@base.UniverseCompatible
class Delete(base.DescribeCommand):
  """Delete a custom role from an organization or a project.

  This command deletes a role.

  This command can fail for the following reasons:
  * The role specified does not exist.
  * The active user does not have permission to access the given role.

  """

  detailed_help = DETAILED_HELP

  @staticmethod
  def Args(parser):
    flags.AddParentFlags(parser, 'delete')
    flags.GetCustomRoleFlag('delete').AddToParser(parser)

  def Run(self, args):
    role_name = iam_util.GetRoleName(args.organization, args.project, args.role)
    client, messages = util.GetClientAndMessages()
    if args.organization is None and args.project is None:
      raise exceptions.InvalidArgumentException(
          'ROLE_ID',
          'You can not delete a curated/predefined role.')
    return client.organizations_roles.Delete(
        messages.IamOrganizationsRolesDeleteRequest(name=role_name))



#!/usr/bin/env python
"""The BigQuery CLI truncate command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from typing import Optional

from absl import app
from absl import flags

import bq_flags
from clients import client_job
from clients import client_table
from clients import utils as bq_client_utils
from frontend import bigquery_command
from frontend import bq_cached_client
from utils import bq_error
from utils import bq_id_utils
from utils import bq_processor_utils

# These aren't relevant for user-facing docstrings:
# pylint: disable=g-doc-return-or-yield
# pylint: disable=g-doc-args


class Truncate(bigquery_command.BigqueryCmd):  # pylint: disable=missing-docstring
  usage = """bq truncate project_id:dataset[.table] [--timestamp] [--dry_run] [--overwrite] [--skip_fully_replicated_tables]
"""
  command = 'truncate'

  def __init__(self, name: str, fv: flags.FlagValues):
    super(Truncate, self).__init__(name, fv)
    flags.DEFINE_integer(
        'timestamp',
        None,
        'Optional timestamp to which table(s) will be truncated. Specified as '
        'milliseconds since epoch.',
        short_name='t',
        flag_values=fv,
    )
    flags.DEFINE_boolean(
        'dry_run',
        None,
        'No-op that simply prints out information and the recommended '
        'timestamp without modifying tables or datasets.',
        flag_values=fv,
    )
    flags.DEFINE_boolean(
        'overwrite',
        False,
        'Overwrite existing tables. Otherwise timestamp will be appended to '
        'all output table names.',
        flag_values=fv,
    )
    flags.DEFINE_boolean(
        'skip_fully_replicated_tables',
        True,
        'Skip tables that are fully replicated (synced) and do not need to be '
        'truncated back to a point in time. This could result in datasets that '
        'have tables synchronized to different points in time, but will '
        'require less data to be re-loaded',
        short_name='s',
        flag_values=fv,
    )

    self._ProcessCommandRc(fv)

  def RunWithArgs(self, identifier: str = '') -> Optional[int]:
    # pylint: disable=g-doc-exception
    """Truncates table/dataset/project to a particular timestamp.

    Examples:
      bq truncate project_id:dataset
      bq truncate --overwrite project_id:dataset --timestamp 123456789
      bq truncate --skip_fully_replicated_tables=false project_id:dataset
    """
    client = bq_cached_client.Client.Get()

    if identifier:
      reference = bq_client_utils.GetReference(
          id_fallbacks=client, identifier=identifier.strip()
      )
    else:
      raise app.UsageError('Must specify one of project, dataset or table')

    self.truncated_table_count = 0
    self.skipped_table_count = 0
    self.failed_table_count = 0
    status = []
    if self.timestamp and not self.dry_run:
      print(
          'Truncating to user specified timestamp %s.(Not skipping fully'
          ' replicated tables.)'
          % self.timestamp
      )
      if isinstance(reference, bq_id_utils.ApiClientHelper.TableReference):
        all_tables = [reference]
      else:
        if isinstance(reference, bq_id_utils.ApiClientHelper.DatasetReference):
          all_tables = list(
              map(
                  lambda x: bq_client_utils.GetReference(
                      id_fallbacks=client, identifier=x['id']
                  ),
                  client_table.list_tables(
                      apiclient=client.apiclient,
                      reference=reference,
                      max_results=1000 * 1000,
                  ),
              )
          )
      for a_table in all_tables:
        try:
          status.append(
              self._TruncateTable(a_table, str(self.timestamp), False)
          )
        except bq_error.BigqueryError as e:
          print(e)
          status.append((self._formatOutputString(a_table, 'Failed')))
          self.failed_table_count += 1
    else:
      if isinstance(reference, bq_id_utils.ApiClientHelper.TableReference):
        all_table_infos = self._GetTableInfo(reference)
      else:
        if isinstance(reference, bq_id_utils.ApiClientHelper.DatasetReference):
          all_table_infos = self._GetTableInfosFromDataset(reference)
      try:
        recovery_timestamp = min(
            list(map(self._GetRecoveryTimestamp, all_table_infos))
        )
      except (ValueError, bq_error.BigqueryTypeError):
        recovery_timestamp = None
      # Error out if we can't figure out a recovery timestamp
      # This can happen in following cases:
      # 1. No multi_site_info present for a table because no commit has been
      #  made to the table.
      # 2. No secondary site is present.
      if not recovery_timestamp:
        raise app.UsageError(
            'Unable to figure out a recovery timestamp for %s. Exiting.'
            % reference
        )
      print('Recommended timestamp to truncate to is %s' % recovery_timestamp)

      for a_table in all_table_infos:
        if not hasattr(reference, 'datasetId'):
          raise AttributeError('Missing `datasetId` on reference.')
        try:
          table_reference = bq_id_utils.ApiClientHelper.TableReference.Create(
              projectId=reference.projectId,
              datasetId=reference.datasetId,
              tableId=a_table['name'],
          )
          status.append(
              self._TruncateTable(
                  table_reference,
                  str(recovery_timestamp),
                  a_table['fully_replicated'],
              )
          )
        except bq_error.BigqueryError as e:
          print(e)
          status.append((self._formatOutputString(table_reference, 'Failed')))
          self.failed_table_count += 1
    print(
        '%s tables truncated, %s tables failed to truncate, %s tables skipped'
        % (
            self.truncated_table_count,
            self.failed_table_count,
            self.skipped_table_count,
        )
    )
    print(*status, sep='\n')

  def _GetTableInfosFromDataset(
      self, dataset_reference: bq_id_utils.ApiClientHelper.DatasetReference
  ):

    # Find minimum of second maximum(latest_replicated_time) for all tables in
    # the dataset and if they are fully replicated.
    recovery_timestamp_for_dataset_query = ("""SELECT
  TABLE_NAME,
  UNIX_MILLIS(replicated_time_at_remote_site),
  CASE
    WHEN last_update_time <= min_latest_replicated_time THEN TRUE
  ELSE
  FALSE
END
  AS fully_replicated
FROM (
  SELECT
    TABLE_NAME,
    multi_site_info.last_update_time,
    ARRAY_AGG(site_info.latest_replicated_time
    ORDER BY
      latest_replicated_time DESC)[safe_OFFSET(1)] AS replicated_time_at_remote_site,
    ARRAY_AGG(site_info.latest_replicated_time
    ORDER BY
      latest_replicated_time ASC)[safe_OFFSET(0)] AS min_latest_replicated_time
  FROM
    %s.INFORMATION_SCHEMA.TABLES t,
    t.multi_site_info.site_info
  GROUP BY
    1,
    2)""") % dataset_reference.datasetId
    return self._ReadTableInfo(
        recovery_timestamp_for_dataset_query, 1000 * 1000
    )

  def _GetTableInfo(
      self, table_reference: bq_id_utils.ApiClientHelper.TableReference
  ):

    # Find second maximum of latest_replicated_time across all sites for this
    # table and if the table is fully replicated
    recovery_timestamp_for_table_query = ("""SELECT
  TABLE_NAME,
  UNIX_MILLIS(replicated_time_at_remote_site),
  CASE
    WHEN last_update_time <= min_latest_replicated_time THEN TRUE
  ELSE
  FALSE
END
  AS fully_replicated
FROM (
  SELECT
    TABLE_NAME,
    multi_site_info.last_update_time,
    ARRAY_AGG(site_info.latest_replicated_time
    ORDER BY
      latest_replicated_time DESC)[safe_OFFSET(1)] AS replicated_time_at_remote_site,
    ARRAY_AGG(site_info.latest_replicated_time
    ORDER BY
      latest_replicated_time ASC)[safe_OFFSET(0)] AS min_latest_replicated_time
  FROM
    %s.INFORMATION_SCHEMA.TABLES t,
    t.multi_site_info.site_info
  WHERE
    TABLE_NAME = '%s'
  GROUP BY
    1,
    2 )""") % (table_reference.datasetId, table_reference.tableId)
    return self._ReadTableInfo(recovery_timestamp_for_table_query, row_count=1)

  def _GetRecoveryTimestamp(self, table_info) -> Optional[int]:
    return (
        int(table_info['recovery_timestamp'])
        if table_info['recovery_timestamp']
        else None
    )

  def _ReadTableInfo(self, query: str, row_count: int):
    client = bq_cached_client.Client.Get()
    try:
      job = client_job.Query(client, query, use_legacy_sql=False)
    except bq_error.BigqueryError as e:
      # TODO(b/324243535): Correct this typing.
      # pytype: disable=attribute-error
      if 'Name multi_site_info not found' in e.error['message']:
        # pytype: enable=attribute-error
        raise app.UsageError(
            'This functionality is not enabled for the current project.'
        )
      else:
        raise e
    all_table_infos = []
    if not bq_client_utils.IsFailedJob(job):
      _, rows = client_job.ReadSchemaAndJobRows(
          client, job['jobReference'], start_row=0, max_rows=row_count
      )
      for i in range(len(rows)):
        table_info = {}
        table_info['name'] = rows[i][0]
        table_info['recovery_timestamp'] = rows[i][1]
        table_info['fully_replicated'] = rows[i][2] == 'true'
        all_table_infos.append(table_info)
      return all_table_infos

  def _formatOutputString(
      self,
      table_reference: bq_id_utils.ApiClientHelper.TableReference,
      status: str,
  ) -> str:
    return '%s %200s' % (table_reference, status)

  def _TruncateTable(
      self,
      table_reference: bq_id_utils.ApiClientHelper.TableReference,
      recovery_timestamp: str,
      is_fully_replicated: bool,
  ) -> str:
    client = bq_cached_client.Client.Get()
    kwds = {}
    if not self.overwrite:
      dest = bq_id_utils.ApiClientHelper.TableReference.Create(
          projectId=table_reference.projectId,
          datasetId=table_reference.datasetId,
          tableId='_'.join(
              [table_reference.tableId, 'TRUNCATED_AT', recovery_timestamp]
          ),
      )
    else:
      dest = table_reference

    if self.skip_fully_replicated_tables and is_fully_replicated:
      self.skipped_table_count += 1
      return self._formatOutputString(
          table_reference, 'Fully replicated...Skipped'
      )
    if self.dry_run:
      return self._formatOutputString(
          dest, 'will be Truncated@%s' % recovery_timestamp
      )
    kwds = {
        'write_disposition': 'WRITE_TRUNCATE',
        'ignore_already_exists': False,
        'operation_type': 'COPY',
    }
    if bq_flags.LOCATION.value:
      kwds['location'] = bq_flags.LOCATION.value
    source_table = bq_client_utils.GetTableReference(
        id_fallbacks=client,
        identifier='%s@%s' % (table_reference, recovery_timestamp),
    )
    job_ref = ' '
    try:
      job = client_job.CopyTable(client, [source_table], dest, **kwds)
      if job is None:
        self.failed_table_count += 1
        return self._formatOutputString(dest, 'Failed')
      job_ref = bq_processor_utils.ConstructObjectReference(job)
      self.truncated_table_count += 1
      return self._formatOutputString(dest, 'Successful %s ' % job_ref)
    except bq_error.BigqueryError as e:
      print(e)
      self.failed_table_count += 1
      return self._formatOutputString(dest, 'Failed %s ' % job_ref)



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to show metadata for a Cloud NetApp Active Directory."""



from googlecloudsdk.api_lib.netapp.active_directories import client as ad_client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.netapp import flags
from googlecloudsdk.command_lib.util.concepts import concept_parsers


@base.ReleaseTracks(base.ReleaseTrack.GA)
class Describe(base.DescribeCommand):
  """Show metadata for a Cloud NetApp Active Directory."""

  _RELEASE_TRACK = base.ReleaseTrack.GA

  detailed_help = {
      'DESCRIPTION': """\
          Describes an AD (Active Directory) config for Cloud NetApp Volumes.
          """,
      'EXAMPLES': """\
          The following command describes an AD named AD_NAME with the required arguments:

              $ {command} AD_NAME --location=us-central1
          """,
  }

  @staticmethod
  def Args(parser):
    concept_parsers.ConceptParser([flags.GetActiveDirectoryPresentationSpec(
        'The Active Directory to describe.')]).AddToParser(parser)

  def Run(self, args):
    """Run the describe command."""
    activedirectory_ref = args.CONCEPTS.active_directory.Parse()
    client = ad_client.ActiveDirectoriesClient(
        release_track=self._RELEASE_TRACK)
    return client.GetActiveDirectory(activedirectory_ref)


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class DescribeBeta(Describe):
  """Show metadata for a Cloud NetApp Active Directory."""

  _RELEASE_TRACK = base.ReleaseTrack.BETA


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class DescribeAlpha(DescribeBeta):
  """Show metadata for a Cloud NetApp Active Directory."""

  _RELEASE_TRACK = base.ReleaseTrack.ALPHA



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Command for accepting spoke updates."""


from googlecloudsdk.api_lib.network_connectivity import networkconnectivity_api
from googlecloudsdk.api_lib.network_connectivity import networkconnectivity_util
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.network_connectivity import flags
from googlecloudsdk.core import log
from googlecloudsdk.core import resources


@base.UniverseCompatible
@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
class AcceptSpokeUpdate(base.Command):
  """Accept a proposal to update a spoke in a hub.

  Accept a proposed or previously rejected VPC spoke update. By accepting a
  spoke update, you permit updating connectivity between the associated VPC
  network and other VPC networks that are attached to the same hub.
  """

  @staticmethod
  def Args(parser):
    flags.AddHubResourceArg(parser, 'to accept the spoke update')
    flags.AddSpokeFlag(parser, 'URI of the spoke to accept update')
    flags.AddSpokeEtagFlag(parser, 'Etag of the spoke to accept update')
    flags.AddAsyncFlag(parser)

  def Run(self, args):
    client = networkconnectivity_api.HubsClient(
        release_track=self.ReleaseTrack()
    )
    hub_ref = args.CONCEPTS.hub.Parse()
    if self.ReleaseTrack() == base.ReleaseTrack.BETA:
      op_ref = client.AcceptSpokeUpdateBeta(
          hub_ref, args.spoke, args.spoke_etag
      )
    else:
      op_ref = client.AcceptSpokeUpdate(hub_ref, args.spoke, args.spoke_etag)

    log.status.Print(
        'Accept spoke update request issued for: [{}]'.format(hub_ref.Name())
    )

    op_resource = resources.REGISTRY.ParseRelativeName(
        op_ref.name,
        collection='networkconnectivity.projects.locations.operations',
        api_version=networkconnectivity_util.VERSION_MAP[self.ReleaseTrack()],
    )
    poller = waiter.CloudOperationPollerNoResources(client.operation_service)

    if op_ref.done:
      return poller.GetResult(op_resource)

    if args.async_:
      log.status.Print('Check operation [{}] for status.'.format(op_ref.name))
      return op_ref

    res = waiter.WaitFor(
        poller, op_resource,
        'Waiting for operation [{}] to complete'.format(op_ref.name))
    return res


AcceptSpokeUpdate.detailed_help = {
    'EXAMPLES':
        """ \
  To accept updating a spoke named ``my-spoke'' with ``etag'' in a hub named ``my-hub'', run:

    $ {command} my-hub --spoke="projects/spoke-project/locations/global/hubs/my-spoke" --spoke-etag=etag
  """,
    'API REFERENCE':
        """ \
  This command uses the networkconnectivity/v1 API. The full documentation
  for this API can be found at:
  https://cloud.google.com/network-connectivity/docs/reference/networkconnectivity/rest
  """,
}



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Describe a version."""


from googlecloudsdk.api_lib.cloudkms import base as cloudkms_base
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.kms import exceptions as kms_exceptions
from googlecloudsdk.command_lib.kms import flags
from googlecloudsdk.core import log
from googlecloudsdk.core.util import files


@base.UniverseCompatible
class Describe(base.DescribeCommand):
  r"""Get metadata for a given version.

  Returns metadata for the given version.

  The optional flag `attestation-file` specifies file to write the attestation
  object into. The attestation object enables the user to verify the integrity
  and provenance of the key. See https://cloud.google.com/kms/docs/attest-key
  for more information about attestations.

  ## EXAMPLES

  The following command returns metadata for version 2 within key `frodo`
  within the keyring `fellowship` in the location `us-east1`:

    $ {command} 2 --key=frodo --keyring=fellowship --location=us-east1

  For key versions with protection level `HSM`, use the `--attestation-file`
  flag to save the attestation to a local file.

    $ {command} 2 --key=frodo --keyring=fellowship --location=us-east1 \
        --attestation-file=path/to/attestation.dat
  """

  @staticmethod
  def Args(parser):
    flags.AddKeyVersionResourceArgument(parser, 'to describe')
    flags.AddAttestationFileFlag(parser)

  def Run(self, args):
    client = cloudkms_base.GetClientInstance()
    messages = cloudkms_base.GetMessagesModule()

    version_ref = flags.ParseCryptoKeyVersionName(args)
    if not version_ref.Name():
      raise exceptions.InvalidArgumentException(
          'version', 'version id must be non-empty.')
    version = client.projects_locations_keyRings_cryptoKeys_cryptoKeyVersions.Get(  # pylint: disable=line-too-long
        messages.
        CloudkmsProjectsLocationsKeyRingsCryptoKeysCryptoKeyVersionsGetRequest(
            name=version_ref.RelativeName()))

    # Raise exception if --attestation-file is provided for software
    # key versions.
    hsm = messages.CryptoKeyVersion.ProtectionLevelValueValuesEnum.HSM
    hsm_st = (
        messages.CryptoKeyVersion.ProtectionLevelValueValuesEnum.HSM_SINGLE_TENANT
    )
    if version.protectionLevel not in [hsm, hsm_st] and args.attestation_file:
      raise kms_exceptions.ArgumentError(
          'Attestations are only available for HSM or HSM_SINGLE_TENANT key '
          'versions.'
      )

    if (args.attestation_file and version.state ==
        messages.CryptoKeyVersion.StateValueValuesEnum.PENDING_GENERATION):
      raise kms_exceptions.ArgumentError(
          'The attestation is unavailable until the version is generated.')

    if args.attestation_file and version.attestation is not None:
      try:
        log.WriteToFileOrStdout(
            args.attestation_file,
            version.attestation.content,
            overwrite=True,
            binary=True)
      except files.Error as e:
        raise exceptions.BadFileException(e)

    if version.attestation is not None:
      # Suppress the attestation content in the printed output. Users can use
      # --attestation-file to obtain it, instead.
      version.attestation.content = None
      # Suppress the attestation content in the printed output. Users can use
      # get-certificate-chain to obtain it, instead.
      version.attestation.certChains = None

    return version



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The command group for gcloud dataflow sql."""


from googlecloudsdk.calliope import base


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
@base.Deprecate(
    is_removed=True,
    warning=(
        'This command is deprecated and will be removed January 31, 2025. '
        'Please see [Beam YAML]'
        '(https://beam.apache.org/documentation/sdks/yaml/) '
        'and [Beam notebooks]'
        '(https://cloud.google.com/dataflow/docs/guides/notebook-advanced#beam-sql) '
        'for alternatives.'
    ),
    error=(
        'This command has been removed. '
        'Please see [Beam YAML]'
        '(https://beam.apache.org/documentation/sdks/yaml/) '
        'and [Beam notebooks]'
        '(https://cloud.google.com/dataflow/docs/guides/notebook-advanced#beam-sql) '
        'for alternatives.'
    ),
)
@base.DefaultUniverseOnly
class Sql(base.Group):
  """A group of subcommands for working with Dataflow SQL."""

  pass



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""List locations."""


from apitools.base.py import list_pager
from googlecloudsdk.api_lib.cloudkms import base as cloudkms_base
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.kms import flags
from googlecloudsdk.core import properties


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA,
                    base.ReleaseTrack.GA)
class List(base.ListCommand):
  """List the project's locations.

  Lists all locations available for this project.
  """

  @staticmethod
  def Args(parser):
    parser.display_info.AddFormat(
        'table(locationId, metadata.hsmAvailable, metadata.ekmAvailable)')
    parser.display_info.AddUriFunc(
        cloudkms_base.MakeGetUriFunc(flags.LOCATION_COLLECTION))

  def Run(self, args):
    client = cloudkms_base.GetClientInstance()
    messages = cloudkms_base.GetMessagesModule()

    project = properties.VALUES.core.project.Get(required=True)
    request = messages.CloudkmsProjectsLocationsListRequest(
        name='projects/' + project)

    return list_pager.YieldFromList(
        client.projects_locations,
        request,
        field='locations',
        limit=args.limit,
        batch_size_attribute='pageSize')



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Implementation of gcloud dataflow jobs drain command.
"""


from googlecloudsdk.api_lib.dataflow import apis
from googlecloudsdk.api_lib.util import exceptions
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.dataflow import job_utils
from googlecloudsdk.core import log


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
@base.DefaultUniverseOnly
class Drain(base.Command):
  """Drains all jobs that match the command line arguments.

     Once Drain is triggered, the pipeline will stop accepting new inputs.
     The input watermark will be advanced to infinity. Elements already in the
     pipeline will continue to be processed. Drained jobs can safely be
     cancelled.
  """

  @staticmethod
  def Args(parser):
    """Register flags for this command."""
    job_utils.ArgsForJobRefs(parser, nargs='+')

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: all the arguments that were provided to this command invocation.
    """
    for job_ref in job_utils.ExtractJobRefs(args):
      try:
        apis.Jobs.Drain(
            job_ref.jobId,
            project_id=job_ref.projectId,
            region_id=job_ref.location)
        log.status.Print('Started draining job [{0}]'.format(job_ref.jobId))
      except exceptions.HttpException as error:
        log.status.Print(
            (
                "Failed to drain job [{0}]: {1} Ensure that you have permission"
                " to access the job and that the `--region` flag, {2}, matches"
                " the job's region."
            ).format(
                job_ref.jobId, error.payload.status_message, job_ref.location
            )
        )



# -*- coding: utf-8 -*- #
# Copyright 2013 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""The gcloud datastore group."""


from googlecloudsdk.calliope import base


@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA,
    base.ReleaseTrack.BETA,
    base.ReleaseTrack.GA,
)
class Datastore(base.Group):
  """Manage your Cloud Datastore resources.

    Cloud Datastore is a highly-scalable NoSQL database for your applications.
    Cloud Datastore automatically handles sharding and replication, providing
    you with a highly available and durable database that scales automatically
    to handle your applications' load.

    More information on Cloud Datastore can be found here:
    https://cloud.google.com/datastore and detailed documentation can be
    found here: https://cloud.google.com/datastore/docs

    export -- Export data to Google Cloud Storage

    import -- Import data from Google Cloud Storage

    indexes -- Manage your Cloud Firestore indexes.

    operations -- Manage Long Running Operations for Cloud Firestore.
  """

  category = base.DATABASES_CATEGORY

  def Filter(self, context, args):
    # TODO(b/190530439):  Determine if command group works with project number
    base.RequireProjectID(args)
    del context, args
    base.DisableUserProjectQuota()



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The command group for versions."""


from googlecloudsdk.api_lib.cloudkms import base as cloudkms_base
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.kms import flags
from googlecloudsdk.core import resources


class Versions(base.Group):
  """Create and manage versions.

  A version represents an individual cryptographic key and the
  associated key material.
  """

  @staticmethod
  def Args(parser):
    parser.display_info.AddUriFunc(
        cloudkms_base.MakeGetUriFunc(flags.CRYPTO_KEY_VERSION_COLLECTION))



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Set IAM workflow template policy command."""


from googlecloudsdk.api_lib.dataproc import dataproc as dp
from googlecloudsdk.api_lib.dataproc import iam_helpers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.dataproc import flags
from googlecloudsdk.command_lib.iam import iam_util


@base.DefaultUniverseOnly
class SetIamPolicy(base.Command):
  """Set IAM policy for a workflow template.

  Sets the IAM policy for a workflow template, given a template ID and the
  policy.
  """

  detailed_help = iam_util.GetDetailedHelpForSetIamPolicy('template')

  @classmethod
  def Args(cls, parser):
    dataproc = dp.Dataproc(cls.ReleaseTrack())
    flags.AddTemplateResourceArg(parser, 'set the policy on',
                                 dataproc.api_version)
    iam_util.AddArgForPolicyFile(parser)

  def Run(self, args):
    dataproc = dp.Dataproc(self.ReleaseTrack())
    msgs = dataproc.messages

    policy = iam_util.ParsePolicyFile(args.policy_file, msgs.Policy)
    policy.version = iam_helpers.MAX_LIBRARY_IAM_SUPPORTED_VERSION
    set_iam_policy_request = msgs.SetIamPolicyRequest(policy=policy)

    template_ref = args.CONCEPTS.template.Parse()
    request = msgs.DataprocProjectsRegionsWorkflowTemplatesSetIamPolicyRequest(
        resource=template_ref.RelativeName(),
        setIamPolicyRequest=set_iam_policy_request)

    return dataproc.client.projects_regions_workflowTemplates.SetIamPolicy(
        request)



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test IAM permissions for a Design Center space."""


from googlecloudsdk.api_lib.design_center import spaces as apis
from googlecloudsdk.api_lib.design_center import utils as api_lib_utils
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.design_center import flags


_DETAILED_HELP = {
    'DESCRIPTION': """ \
        Tests the IAM permissions that a caller has on a Design Center space.

        Returns an empty set of permissions if the space does not exist.

        Note: This operation is designed to be used for building
        permission-aware UIs and command-line tools, not for authorization
        checking. This operation may "fail open" without warning.
        """,
    'EXAMPLES': """ \
        To test if the caller has the `designcenter.spaces.get` and
        `designcenter.spaces.update` permissions on the Space `my-space` in
        project `my-project` and location `us-central1`, run:

          $ {command} my-space --location=us-central1 --project=my-project \
              --permissions=designcenter.spaces.get,designcenter.spaces.update
        """,
    'API REFERENCE': """ \
        This command uses the designcenter/v1alpha API. The full documentation for
        this API can be found at:
        http://cloud.google.com/application-design-center/docs
        """,
}


@base.ReleaseTracks(base.ReleaseTrack.GA)
@base.UniverseCompatible
class TestIamPermissionsGA(base.Command):
  """Test IAM permissions for a Design Center space."""
  detailed_help = _DETAILED_HELP

  @staticmethod
  def Args(parser):
    flags.AddTestIamPermissionsFlags(parser)
    parser.add_argument(
        '--permissions',
        metavar='PERMISSION',
        type=arg_parsers.ArgList(),
        required=True,
        help='The set of permissions to check for the resource.',
    )

  def Run(self, args):
    client = apis.SpacesClient(release_track=base.ReleaseTrack.GA)
    space_ref = api_lib_utils.GetSpaceRef(args)
    return client.TestIamPermissions(
        space_id=space_ref.RelativeName(),
        permissions=args.permissions,
    )


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@base.UniverseCompatible
class TestIamPermissions(base.Command):
  """Test IAM permissions for a Design Center space."""
  detailed_help = _DETAILED_HELP

  @staticmethod
  def Args(parser):
    flags.AddTestIamPermissionsFlags(parser)
    parser.add_argument(
        '--permissions',
        metavar='PERMISSION',
        type=arg_parsers.ArgList(),
        required=True,
        help='The set of permissions to check for the resource.',
    )

  def Run(self, args):
    client = apis.SpacesClient(release_track=base.ReleaseTrack.ALPHA)
    space_ref = api_lib_utils.GetSpaceRef(args)
    return client.TestIamPermissions(
        space_id=space_ref.RelativeName(),
        permissions=args.permissions,
    )



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Wait operations command."""


from apitools.base.py import exceptions as apitools_exceptions

from googlecloudsdk.api_lib.container import util
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions


class Wait(base.Command):
  """Poll an operation for completion."""

  detailed_help = {
      'DESCRIPTION':
          '{description}',
      'EXAMPLES':
          """\
          To poll an operation for completion, run:

            $ {command} sample-operation-id
          """,
  }

  @staticmethod
  def Args(parser):
    """Register flags for this command.

    Args:
      parser: An argparse.ArgumentParser-like object. It is mocked out in order
        to capture some information, but behaves like an ArgumentParser.
    """
    parser.add_argument('operation_id', help='The operation id to poll.')

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Returns:
      Some value that we want to have printed later.
    """
    adapter = self.context['api_adapter']
    location_get = self.context['location_get']
    location = location_get(args)

    operation_ref = adapter.ParseOperation(args.operation_id, location)

    try:
      # Try fetching it first to ensure we have permissions and the op exists.
      adapter.GetOperation(operation_ref)

      return adapter.WaitForOperation(
          operation_ref,
          'Waiting for {0} to complete'.format(operation_ref.operationId))
    except apitools_exceptions.HttpError as error:
      raise exceptions.HttpException(error, util.HTTP_ERROR_FORMAT)



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Get the public key for a given version."""


from googlecloudsdk.api_lib.cloudkms import base as cloudkms_base
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.kms import flags
from googlecloudsdk.command_lib.kms import maps
from googlecloudsdk.core import log


@base.UniverseCompatible
class GetPublicKey(base.DescribeCommand):
  r"""Get the public key for a given version.

  Returns the public key of the given asymmetric key version in the specified format.

  The optional flag `output-file` indicates the path to store the public key.
  If not specified, the public key will be printed to stdout.

  The optional flag `public-key-format` indicates the format in which the
  public key will be returned. For the NIST PQC algorithms, this must be
  specified and set to `nist-pqc`. For kem-xwing this must be specified and set to
  `xwing-raw-bytes`. For all other algorithms, this flag is
  optional and can be either `pem` or `der`; the default value is `pem`.
  See "Retrieve a public key" in the Cloud KMS
  documentation (https://cloud.google.com/kms/help/get-public-key) for more
  information about the supported formats.

  ## EXAMPLES

  The following command saves the public key for CryptoKey `frodo` Version 2
  to '/tmp/my/public_key.file':

    $ {command} 2 \
    --key=frodo \
    --keyring=fellowship \
    --location=us-east1 \
    --public-key-format=pem \
    --output-file=/tmp/my/public_key.file
  """

  @staticmethod
  def Args(parser):
    flags.AddKeyVersionResourceArgument(parser, 'to get public key')
    flags.AddOutputFileFlag(parser, 'to store public key')
    flags.AddPublicKeyFormatFlag(parser)

  def Run(self, args):
    client = cloudkms_base.GetClientInstance()
    messages = cloudkms_base.GetMessagesModule()

    version_ref = flags.ParseCryptoKeyVersionName(args)
    if not version_ref.Name():
      raise exceptions.InvalidArgumentException(
          'version', 'version id must be non-empty.'
      )
    req = messages.CloudkmsProjectsLocationsKeyRingsCryptoKeysCryptoKeyVersionsGetPublicKeyRequest(  # pylint: disable=line-too-long
        name=version_ref.RelativeName(),
    )
    if args.public_key_format:
      if args.public_key_format not in [
          'pem',
          'der',
          'nist-pqc',
          'xwing-raw-bytes',
      ]:
        raise exceptions.InvalidArgumentException(
            '--public-key-format',
            'Invalid value for public key format. Supported values are "der",'
            ' "pem", "xwing-raw-bytes" and "nist-pqc".',
        )
      req.publicKeyFormat = maps.PUBLIC_KEY_FORMAT_MAPPER.GetEnumForChoice(
          args.public_key_format
      )
    resp = client.projects_locations_keyRings_cryptoKeys_cryptoKeyVersions.GetPublicKey(
        req
    )

    # DER, NIST_PQC and XWING_RAW_BYTES are in raw byte format.
    # So the output is in binary format.
    write_binary_file = args.public_key_format in [
        'der',
        'nist-pqc',
        'xwing-raw-bytes',
    ]
    log.WriteToFileOrStdout(
        args.output_file if args.output_file else '-',
        resp.pem if resp.pem else resp.publicKey.data,
        overwrite=True,
        binary=write_binary_file,
        private=True,
    )



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Describe deployment command."""


from googlecloudsdk.api_lib.network_security.mirroring_deployments import api
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.network_security import deployment_flags

DETAILED_HELP = {
    'DESCRIPTION': """
          Describe a mirroring deployment.

          For more examples, refer to the EXAMPLES section below.

        """,
    'EXAMPLES': """
            To get a description of a mirroring deployment called `my-deployment` in
            zone `us-central1-a`, run:

            $ {command} my-deployment --location=us-central1-a --project=my-project

            OR

            $ {command} projects/my-project/locations/us-central1-a/mirroringDeployments/my-deployment
        """,
}


@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA,
                    base.ReleaseTrack.GA)
class Describe(base.DescribeCommand):
  """Describe a Mirroring Deployment."""

  @classmethod
  def Args(cls, parser):
    deployment_flags.AddDeploymentResource(cls.ReleaseTrack(), parser)

  def Run(self, args):
    client = api.Client(self.ReleaseTrack())

    deployment = args.CONCEPTS.mirroring_deployment.Parse()

    return client.DescribeDeployment(name=deployment.RelativeName())


Describe.detailed_help = DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""gcloud dns operations describe command."""


from googlecloudsdk.api_lib.dns import operations
from googlecloudsdk.api_lib.dns import util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.dns import flags
from googlecloudsdk.core import properties


def _CommonArgs(parser):
  flags.GetZoneArg('Name of zone to get operations from.').AddToParser(parser)
  parser.add_argument('operation_id', metavar='OPERATION_ID',
                      help='The id of the operation to display.')


def _Describe(operations_client, args):
  operation_ref = util.GetRegistry(operations_client.version).Parse(
      args.operation_id,
      params={
          'project': properties.VALUES.core.project.GetOrFail,
          'managedZone': args.zone
      },
      collection='dns.managedZoneOperations')

  return operations_client.Get(operation_ref)


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class DescribeBeta(base.DescribeCommand):
  """Describe an operation.

  This command displays the details of a single managed-zone operation.

  ## EXAMPLES

  To describe a managed-zone operation:

    $ {command} 1234 --zone=my_zone
  """

  @staticmethod
  def Args(parser):
    _CommonArgs(parser)

  def Run(self, args):
    api_version = util.GetApiFromTrack(self.ReleaseTrack())
    operations_client = operations.Client.FromApiVersion(api_version)
    return _Describe(operations_client, args)


@base.ReleaseTracks(base.ReleaseTrack.GA)
class Describe(base.DescribeCommand):
  """Describe an operation.

  This command displays the details of a single managed-zone operation.

  ## EXAMPLES

  To describe a managed-zone operation:

    $ {command} 1234 --zone=my_zone
  """

  @staticmethod
  def Args(parser):
    _CommonArgs(parser)

  def Run(self, args):
    operations_client = operations.Client.FromApiVersion('v1')
    return _Describe(operations_client, args)


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class DescribeAlpha(DescribeBeta):
  """Describe an operation.

  This command displays the details of a single managed-zone operation.

  ## EXAMPLES

  To describe a managed-zone operation:

    $ {command} 1234 --zone=my_zone
  """



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""List keyhandles within a project and location."""


from apitools.base.py import list_pager
from googlecloudsdk.api_lib.cloudkms import base as cloudkms_base
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.kms import flags
from googlecloudsdk.command_lib.kms import resource_args


@base.UniverseCompatible
class List(base.ListCommand):
  """List KeyHandle resources within a project and location.

  Lists all KeyHandle resources within a given project and location.
  Addtionally, can filter the list.

  ## EXAMPLES

  The following command lists a maximum of five KeyHandle resources in the
  location `global`:

    $ {command} --location=global --limit=5

  The following command lists all KeyHandle resources in the location `global`
  that have a resource type selector of `compute.googleapis.com/Instance`:

    $ {command} --location=global
    --resource-type=compute.googleapis.com/Instance
  """

  @staticmethod
  def Args(parser):
    resource_args.AddKmsLocationResourceArgForKMS(parser, True, '--location')
    flags.AddResourceTypeSelectorFlag(parser, True)

    parser.display_info.AddFormat('table(name, kmsKey, resourceTypeSelector)')

  def Run(self, args):
    client = cloudkms_base.GetClientInstance()
    messages = cloudkms_base.GetMessagesModule()
    location_ref = args.CONCEPTS.location.Parse()

    filter_str = f'resource_type_selector="{args.resource_type}"'

    request = messages.CloudkmsProjectsLocationsKeyHandlesListRequest(
        parent=location_ref.RelativeName(), filter=filter_str
    )

    return list_pager.YieldFromList(
        client.projects_locations_keyHandles,
        request,
        field='keyHandles',
        limit=args.limit,
        batch_size_attribute='pageSize',
    )



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Implementation of gcloud dataflow jobs update-options command."""


from googlecloudsdk.api_lib.dataflow import apis
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.dataflow import job_utils


@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA)
class UpdateOptions(base.Command):
  """Update pipeline options on-the-fly for running Dataflow jobs.

  This command can modify properties of running Dataflow jobs. Currently, only
  updating autoscaling settings for Streaming Engine jobs is supported.

  Adjust the autoscaling settings for Streaming Engine Dataflow jobs by
  providing at-least one of --min-num-workers or --max-num-workers or
  --worker-utilization-hint (or all 3), or --unset-worker-utilization-hint
  (which cannot be run at the same time as --worker-utilization-hint but works
  with the others).
  Allow a few minutes for the changes to take effect.

  Note that autoscaling settings can only be modified on-the-fly for Streaming
  Engine jobs. Attempts to modify batch job or Streaming Appliance jobs will
  fail.


  ## EXAMPLES

  Modify autoscaling settings to scale between 5-10 workers:

    $ {command} --min-num-workers=5 --max-num-workers=10

  Require a job to use at least 2 workers:

    $ {command} --min-num-workers=2

  Require a job to use at most 20 workers:

    $ {command} --max-num-workers=20

  Adjust the hint of target worker utilization to 70% for horizontal
  autoscaling:

    $ {command} --worker-utilization-hint=0.7

  "Unset" worker utilization hint so that horizontal scaling will rely on its
  default CPU utilization target:

    $ {command} --unset-worker-utilization-hint
  """

  @staticmethod
  def Args(parser):
    """Register flags for this command."""
    job_utils.ArgsForJobRef(parser)
    parser.add_argument(
        '--min-num-workers',
        type=int,
        help=(
            'Lower-bound for autoscaling, between 1-1000. Only supported for'
            ' streaming-engine jobs.'
        ),
    )
    parser.add_argument(
        '--max-num-workers',
        type=int,
        help=(
            'Upper-bound for autoscaling, between 1-1000. Only supported for'
            ' streaming-engine jobs.'
        ),
    )
    parser.add_argument(
        '--worker-utilization-hint',
        type=float,
        help=(
            'Target CPU utilization for autoscaling, ranging from 0.1 to 0.9.'
            ' Only supported for streaming-engine jobs with autoscaling'
            ' enabled.'
        ),
    )
    parser.add_argument(
        '--unset-worker-utilization-hint',
        action='store_true',
        help=(
            'Unset --worker-utilization-hint. This causes the'
            ' job autoscaling to fall back to internal tunings'
            ' if they exist, or otherwise use the default hint value.'
        ),
    )

  def Run(self, args):
    """Called when the user runs gcloud dataflow jobs update-options ...

    Args:
      args: all the arguments that were provided to this command invocation.

    Returns:
      The updated Job
    """
    if (
        args.min_num_workers is None
        and args.max_num_workers is None
        and args.worker_utilization_hint is None
        and not args.unset_worker_utilization_hint
    ):
      raise exceptions.OneOfArgumentsRequiredException(
          [
              '--min-num-workers',
              '--max-num-workers',
              '--worker-utilization-hint',
              '--unset-worker-utilization-hint',
          ],
          'You must provide at-least one field to update',
      )
    elif (
        args.worker_utilization_hint is not None
        and args.unset_worker_utilization_hint
    ):
      raise exceptions.ConflictingArgumentsException(
          'The arguments --worker-utilization-hint and'
          ' --unset-worker-utilization-hint are mutually exclusive (as the'
          ' unset command will unset the given hint), and must be called'
          ' separately.',
      )

    job_ref = job_utils.ExtractJobRef(args)
    return apis.Jobs.UpdateOptions(
        job_ref.jobId,
        project_id=job_ref.projectId,
        region_id=job_ref.location,
        min_num_workers=args.min_num_workers,
        max_num_workers=args.max_num_workers,
        worker_utilization_hint=args.worker_utilization_hint,
        unset_worker_utilization_hint=args.unset_worker_utilization_hint,
    )



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""The command group for the MPS CLI."""


from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.projects import util


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class Mps(base.Group):
  """Manage Marketplace Solutions resources."""

  category = base.COMPUTE_CATEGORY

  @staticmethod
  def Args(parser):
    parser.display_info.AddUriFunc(util.ProjectsUriFunc)

  def Filter(self, context, args):
    del context, args
    # Don't ever take this off. Use gcloud quota for projects operations so
    # you can create a project before you have a project.
    base.DisableUserProjectQuota()



# -*- coding: utf-8 -*- #
# Copyright 2024 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command group for Submitting Spark Applciations on Dataproc GDC service instances."""


from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base


@base.Hidden
@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.GA)
class SparkApplications(base.Group):
  """Submit Dataproc GDC Spark Applications."""

  category = base.DATA_ANALYTICS_CATEGORY

  @staticmethod
  def Args(parser):
    parser.add_argument(
        '--dependency-images',
        help=(
            'Comma separated list of images containing dependencies for the'
            ' Spark Application.'
        ),
        metavar='IMAGE',
        type=arg_parsers.ArgList(),
    )



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The command to deploy or update the Cloud Run for Anthos feature."""


from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.container.fleet import kube_util
from googlecloudsdk.command_lib.container.fleet import util as hub_util
from googlecloudsdk.core import exceptions
from googlecloudsdk.core import log
from googlecloudsdk.core import yaml
from googlecloudsdk.core.util import files

# Pull out the example text so the example command can be one line without the
# py linter complaining. The docgen tool properly breaks it into multiple lines.
EXAMPLES = """\
    To apply the CloudRun YAML file, run:

      $ {command} --kubeconfig=/path/to/kubeconfig  --config=/path/to/cloud-run-cr.yaml
"""


class Apply(base.CreateCommand):
  """Deploy or update the CloudRun feature.

  Deploy or update a user-specified config file of the CloudRun custom resource.
  The config file should be a YAML file.
  """

  detailed_help = {'EXAMPLES': EXAMPLES}

  feature_name = 'appdevexperience'

  @staticmethod
  def Args(parser):
    hub_util.AddClusterConnectionCommonArgs(parser)

    parser.add_argument(
        '--config',
        type=str,
        help='The path to CloudRun custom resource config file.',
        required=False)

  def Run(self, args):
    kube_client = kube_util.KubernetesClient(
        gke_uri=getattr(args, 'gke_uri', None),
        gke_cluster=getattr(args, 'gke_cluster', None),
        kubeconfig=getattr(args, 'kubeconfig', None),
        context=getattr(args, 'context', None),
        public_issuer_url=getattr(args, 'public_issuer_url', None),
        enable_workload_identity=getattr(args, 'enable_workload_identity',
                                         False),
    )
    kube_util.ValidateClusterIdentifierFlags(kube_client, args)

    yaml_string = files.ReadFileContents(
        args.config) if args.config is not None else _default_cr()

    _validate_cr(yaml_string)

    _apply_cr_to_membership_cluster(kube_client, yaml_string)

    log.status.Print('Added CloudRun CR')


def _apply_cr_to_membership_cluster(kube_client, yaml_string):
  """Apply the CloudRun custom resource to the cluster.

  Args:
    kube_client: A Kubernetes client.
    yaml_string: the CloudRun YAML file.
  """
  _, err = kube_client.Apply(yaml_string)
  if err:
    raise exceptions.Error(
        'Failed to apply manifest to cluster: {}'.format(err))


def _validate_cr(yaml_string):
  """Validate the parsed cloudrun YAML.

  Args:
    yaml_string: The YAML string to validate.
  """

  try:
    cloudrun_cr = yaml.load(yaml_string)
  except yaml.Error as e:
    raise exceptions.Error('Invalid cloudrun yaml {}'.format(yaml_string), e)

  if not isinstance(cloudrun_cr, dict):
    raise exceptions.Error('Invalid CloudRun template.')
  if 'apiVersion' not in cloudrun_cr:
    raise exceptions.Error(
        'The resource is missing a required field "apiVersion".')
  if cloudrun_cr['apiVersion'] != 'operator.run.cloud.google.com/v1alpha1':
    raise exceptions.Error(
        'The resource "apiVersion" field must be set to: "operator.run.cloud.google.com/v1alpha1". If you believe the apiVersion is correct, you may need to upgrade your gcloud installation.'
    )

  if 'kind' not in cloudrun_cr:
    raise exceptions.Error('The resource is missing a required field "kind".')

  if cloudrun_cr['kind'] != 'CloudRun':
    raise exceptions.Error(
        'The resource "kind" field must be set to: "CloudRun".')

  if 'metadata' not in cloudrun_cr:
    raise cloudrun_cr.Error(
        'The resource is missing a required field "metadata".')

  metadata = cloudrun_cr['metadata']
  if ('name' not in metadata or metadata['name'] != 'cloud-run'):
    raise exceptions.Error(
        'The resource "metadata.name" field must be set to "cloud-run"')


def _default_cr():
  return r"""
  apiVersion: operator.run.cloud.google.com/v1alpha1
  kind: CloudRun
  metadata:
    name: cloud-run
  """



# -*- coding: utf-8 -*- #
# Copyright 2019 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to wait for Cloud Life Sciences operation to complete."""


from googlecloudsdk.api_lib.lifesciences import lifesciences_client
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.command_lib.lifesciences import operation_poller
from googlecloudsdk.command_lib.util.apis import yaml_data
from googlecloudsdk.command_lib.util.concepts import concept_parsers


class Wait(base.SilentCommand):
  r"""Wait for Cloud Life Sciences operation to complete.

  ## EXAMPLES
  To wait for the completion of the operation called `my-operation`, run:

    $ {command} my-operation
  """

  WAIT_CEILING_MS = 60 * 20 * 1000

  @staticmethod
  def Args(parser):
    operation_spec = concepts.ResourceSpec.FromYaml(
        yaml_data.ResourceYAMLData.FromPath('lifesciences.operation')
        .GetData())
    concept_parsers.ConceptParser.ForResource(
        'operation', operation_spec, 'The Cloud Life Sciences operation to wait for.',
        required=True).AddToParser(parser)

  def Run(self, args):
    client = lifesciences_client.LifeSciencesClient()
    operation_ref = args.CONCEPTS.operation.Parse()

    req = client.messages.LifesciencesProjectsLocationsOperationsGetRequest(
        name=operation_ref.RelativeName())

    operation = client.client.projects_locations_operations.Get(req)

    waiter.WaitFor(
        operation_poller.OperationPoller(),
        operation.name,
        'Waiting for [{}] to complete.'.format(operation.name),
        wait_ceiling_ms=self.WAIT_CEILING_MS)



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to describe a trigger."""


from googlecloudsdk.api_lib.eventarc import triggers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.eventarc import flags
from googlecloudsdk.core import log

_DETAILED_HELP = {
    'DESCRIPTION':
        '{description}',
    'EXAMPLES':
        """ \
        To describe the trigger ``my-trigger'', run:

          $ {command} my-trigger
        """,
}


@base.ReleaseTracks(base.ReleaseTrack.GA)
@base.DefaultUniverseOnly
class Describe(base.DescribeCommand):
  """Describe an Eventarc trigger."""

  detailed_help = _DETAILED_HELP

  @staticmethod
  def Args(parser):
    flags.AddTriggerResourceArg(
        parser, 'The trigger to describe.', required=True)

  def Run(self, args):
    """Run the describe command."""
    client = triggers.TriggersClientV1()
    trigger_ref = args.CONCEPTS.trigger.Parse()
    trigger = client.Get(trigger_ref)
    event_type = client.GetEventType(trigger)
    self._active_time = triggers.TriggerActiveTime(event_type,
                                                   trigger.updateTime)
    return trigger

  def Epilog(self, resources_were_displayed):
    if resources_were_displayed and self._active_time:
      log.warning(
          'The trigger was recently modified and will become active by {}.'
          .format(self._active_time))



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Command to list autonomous database backups."""


from apitools.base.py import list_pager
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.command_lib.util.concepts import concept_parsers

VERSION_MAP = {
    base.ReleaseTrack.ALPHA: 'v1alpha',
    base.ReleaseTrack.GA: 'v1',
}


def LocationAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='location', help_text='The Cloud location for the {resource}.')


def GetLocationResourceSpec(resource_name='location'):
  return concepts.ResourceSpec(
      'oracledatabase.projects.locations',
      resource_name=resource_name,
      locationsId=LocationAttributeConfig(),
      projectsId=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      disable_auto_completers=True)


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.GA)
@base.DefaultUniverseOnly
class List(base.ListCommand):
  r"""List autonomous database backups.

  ## DESCRIPTION
    Lists all AutonomousDatabaseBackups for the specified
    AutonomousDatabase.

  ## EXAMPLES
  To list all backups for an AutonomousDatabase with id `my-instance` in the
  location `us-east4`, run:

      $ {command} --location=us-east4
          --filter='autonomous_database_id="my-instance"'
  """

  @staticmethod
  def Args(parser):
    concept_parsers.ConceptParser.ForResource(
        '--location',
        GetLocationResourceSpec(),
        group_help='The location you want to list the connection profiles for.',
        required=True).AddToParser(parser)

    base.FILTER_FLAG.RemoveFromParser(parser)
    base.Argument(
        '--filter',
        metavar='EXPRESSION',
        require_coverage_in_tests=False,
        category=base.LIST_COMMAND_FLAGS,
        help="""\
        Apply a ADB filter in the format :
        --filter='autonomous_database_id="my-instance"'
        """,
    ).AddToParser(parser)

  def Run(self, args):
    """List autonomous database backups."""
    client = apis.GetClientInstance(
        'oracledatabase', VERSION_MAP[self.ReleaseTrack()])
    messages = apis.GetMessagesModule(
        'oracledatabase', VERSION_MAP[self.ReleaseTrack()])
    ref = args.CONCEPTS.location.Parse()
    server_filter = args.filter
    # TODO(b/366489468): Remap the filter logic so that it works for both client
    # and server side filter side.
    # args.filter needs to be set to '' because the response doesn't contains
    # autonomous_database_id in the response, so if the client side filter
    # is not empty, the output will be empty.
    args.filter = ''
    return list_pager.YieldFromList(
        client.projects_locations_autonomousDatabaseBackups,
        messages.OracledatabaseProjectsLocationsAutonomousDatabaseBackupsListRequest(
            parent=ref.RelativeName(),
            pageSize=args.page_size,
            filter=server_filter,
        ),
        batch_size=args.page_size,
        field='autonomousDatabaseBackups',
        limit=args.limit,
        batch_size_attribute='pageSize',
    )



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""service-management operations wait command."""


from googlecloudsdk.api_lib.endpoints import services_util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.endpoints import arg_parsers
from googlecloudsdk.command_lib.endpoints import common_flags


class Wait(base.Command):
  # pylint: disable=line-too-long
  """Waits for an operation to complete.

     This command will block until an operation has been marked as complete.

     Note that the `operations/` prefix of the operation name is optional
     and may be omitted.

     ## EXAMPLES
     To wait on an operation named `operations/serviceConfigs.my-service.1`
     to complete, run:

       $ {command} serviceConfigs.my-service.1
  """
  # pylint: enable=line-too-long

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command.

    Args:
      parser: An argparse parser that you can use to add arguments that go
          on the command line after this command. Positional arguments are
          allowed.
    """
    common_flags.operation_flag(suffix='on which to wait').AddToParser(parser)

  def Run(self, args):
    """Run 'service-management operations wait'.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
          with.

    Returns:
      If successful, the response from the operations.Get API call.
    """
    messages = services_util.GetMessagesModule()
    client = services_util.GetClientInstance()

    operation_id = arg_parsers.GetOperationIdFromArg(args.operation)

    request = messages.ServicemanagementOperationsGetRequest(
        operationsId=operation_id,)

    operation = client.operations.Get(request)

    return services_util.ProcessOperationResult(operation, is_async=False)



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The `gcloud domains registrations operations describe` command."""


from googlecloudsdk.api_lib.domains import operations
from googlecloudsdk.api_lib.domains import registrations
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.domains import resource_args


@base.DefaultUniverseOnly
class Describe(base.DescribeCommand):
  """Show details about a Cloud Domains operation.

  Print information about a Cloud Domains operation.

  ## EXAMPLES

  To describe an operation ``operation-id'', run:

    $ {command} operation-id
  """

  @staticmethod
  def Args(parser):
    resource_args.AddOperationResourceArg(parser, 'to describe')

  def Run(self, args):
    api_version = registrations.GetApiVersionFromArgs(args)
    client = operations.Client.FromApiVersion(api_version)
    operation_ref = args.CONCEPTS.operation.Parse()
    return client.Get(operation_ref)
